\name{MCDIST}
\alias{MCDIST}
\title{
Analysis of Chromatin Interaction Using MC_DIST Model
}
\description{
Given count, marginal count and distance information, plus the tuning parameters for the MCMC algorithm,
the MCDIST function returns the posterior probability of being a true interaction for each observation, 
acceptance rates of the parameters and the stored samples for the parameters.
}
\usage{
MCDIST(count,rmcd,
  init1=list("z"=rep(1,length(count)),
    "lambda1"=rep(model$cut.off+5.0,length(count)),
    "lambda0"=rep(model$cut.off+1.0,length(count)),
    "pi"=rep(0.5,length(count))),
  init2=list("d"=0.1,"sigma2"=1.0,"r0"=1.0,"b"=1.0),
  proposal=list("d"=0.1,"r0"=0.1,"b"=0.1,"lambda1"=0.1,"lambda0"=0.1),
  model=list("cut.off"=1,"UUB"=1000,"VUB"=1000^2,"DUB"=10000),
  running=list("iter"=1000,"thin"=10,"burnin"=200,"seed"=2000),
  monitor=list(
  "obs1"=match(min(abs(count-quantile(count)[2])),
               abs(count-quantile(count)[2])),
  "obs2"=match(min(abs(count-quantile(count)[3])),
               abs(count-quantile(count)[3])),
  "obs3"=match(min(abs(count-quantile(count)[4])),
               abs(count-quantile(count)[4]))))
}
\author{
Liang Niu
}
\arguments{
  \item{count}{a vector specifying the observation (count) to be analyzed.}
  \item{rmcd}{a vector specifying the ratio of marginal count to distance for each observation, with the same length as count.}
  \item{init1}{an optional list of four numeric vectors with names "z", "lambda1","lambda0" and "pi". Each vector is of length n and gives the initial values of the corresponding parameter.}
  \item{init2}{an optional list of four numeric values with names "d", "sigma2" (sigma square), "r0" and "b". Each value is the initial value of the corresponding parameter.}  
  \item{proposal}{an optional list of five numeric values with names "d", "r0", "b", "lambda1" and "lambda0". Each value is the proposal standard deviation for the corresponding parameter.}
  \item{model}{an optional list of four numeric values with names "cut.off" (cut off value for data), "UUB" (upperbound of uniform distributions for r0 and b), "VUB" (square of the upperbound of uniform distributions for sigma) and "DUB" (upperbound of uniform distribution for d), which are the model setting parameters.}
  \item{running}{an optional list of four numeric values with names "iter" (number of iterations), "thin" (thinning number), "burnin" (burnin number) and "seed" (random seed), which are the parameters to set up MCMC. }
  \item{monitor}{an optional list of three integers with names "obs1", "obs2" and "obs3", which are the indices of the three monitored observations. The default are the three observations with counts closest to the 25\%, 50\% and 75\% percentile of the data, respectively. In case of multiple matches, the first one in the data set is used.}
}
\value{
  A list including elements
  \item{result}{a dataframe containg the analysis result. It has two columns with names "posterior.probability" and "interaction.type".}
  \item{acceptance}{a list of acceptance rates with names "r0", "b", "d", "lambda1.min", "lambda1.max", "lambda0.min" and "lambda0.max". The last four values give the two ranges of acceptance rates for lambda1 and lambda0.}
  \item{samples}{a list of 13 numeric vectors of stored samples (after thining) with names "d", "sigma2" (sigma square), "r0", "b", "pi_obs1", "lambda1_obs1", "lambda0_obs1", "pi_obs2", "lambda1_obs2", "lambda0_obs2", "pi_obs3", "lambda1_obs3", "lambda0_obs3", the last nine vectors give the samples of pi, lambda1 and lambda0 for the three monitored observations.}
  \item{computing.time}{a vaule of computing time, in unit of seconds.}
}
\examples{
data("simdata")
n=length(simdata$count)
output=MCDIST(count=simdata$count,rmcd=simdata$mc/simdata$distance,
              init1=list("z"=rep(1,n),"lambda1"=rep(30.0,n),
                         "lambda0"=rep(1.0,n),"pi"=rep(0.5,n)),
              init2=list("d"=0.5,"sigma2"=10.0,"r0"=5.0,"b"=1.0),
              proposal=list("d"=0.036,"r0"=0.015,"b"=0.054,
                            "lambda1"=0.6,"lambda0"=1.1))
output$result$interaction.type
}


