\name{MDOS}
\alias{MDOS}
\title{
One-Step Model
}
\description{
The one-step model for detecting chromatin interactions with different intensities in two samples. 
}
\usage{
MDOS(count.1,count.2,dirichlet.const,
        init1=list("z"=rep(1,length(count.1)),
                "lambda0"=rep(model$cut.off+10.0,length(count.1)),
                "lambda1"=rep(model$cut.off+5.0,length(count.1)),
                "lambda2"=rep(model$cut.off+5.0,length(count.1)),
                "lambda3"=rep(model$cut.off+10.0,length(count.1)),
                "lambda4"=rep(model$cut.off+1.0,length(count.1)),
                "lambda5"=rep(model$cut.off+1.0,length(count.1)),
                "mu0"=rep(model$cut.off+5.0,length(count.1)),
                "mu2"=rep(model$cut.off+10.0,length(count.1)),
                "mu3"=rep(model$cut.off+1.0,length(count.1)),
                "mu4"=rep(model$cut.off+10.0,length(count.1)),
                "mu5"=rep(model$cut.off+1.0,length(count.1)),
                "w0"=rep(1/6,length(count.1)),
                "w1"=rep(1/6,length(count.1)),
                "w2"=rep(1/6,length(count.1)),
                "w3"=rep(1/6,length(count.1)),
                "w4"=rep(1/6,length(count.1)),
                "w5"=rep(1/6,length(count.1))),
        init2=list("d"=1.0,"sigma2"=1.0,"r0"=1.0,"b"=1.0),
        proposal=list("d"=1.0,"r0"=0.1,"b"=0.1,
                      "lambda0"=0.1,"lambda1"=0.1,
                      "lambda2"=0.1,"lambda3"=0.1,
                      "lambda4"=0.1,"lambda5"=0.1,
                      "mu0"=0.1,"mu2"=0.1,"mu3"=0.1,
                      "mu4"=0.1,"mu5"=0.1),
        model=list("cut.off"=1,"UUB"=1000,"VUB"=1000^2,"DUB"=100000),
        running=list("iter"=1000,"thin"=10,"burnin"=200,"seed"=2000),
        monitor=list("obs1"=match(min(abs(count.1-quantile(count.1)[2])),
                                  abs(count.1-quantile(count.1)[2])),
                     "obs2"=match(min(abs(count.1-quantile(count.1)[3])),
                                  abs(count.1-quantile(count.1)[3])),
                     "obs3"=match(min(abs(count.1-quantile(count.1)[4])),
                                  abs(count.1-quantile(count.1)[4]))))}
\author{
Liang Niu
}
\arguments{
  \item{count.1}{a vector specifying the observations (counts)of DNA fragment pairs claimed as true pairs (pairs formed by true chromatin interactions)in sample one by MC_DIST model (the model in the first step of the two-step model).}
  \item{count.2}{a vector specifying the observations (counts)of DNA fragment pairs claimed as true pairs (pairs formed by true chromatin interactions)in sample two by MC_DIST model (the model in the first step of the two-step model).}
  \item{dirichlet.const}{a list of six numeric vectors with names "c0", "c1", "c2", "c3", "c4" and "c5". Each vector is of length n (length of count.1 and count.2) and specifies the corresponding constant factors in the dirichlet distributions for the weights of the six components of the model.}
  \item{init1}{an optional list of eighteen numeric vectors with names "z", "lambda0", "lambda1", "lambda2", "lambda3", "lambda4", "lambda5","mu0", "mu2", "mu3", "mu4", "mu5","w0", "w1", "w2","w3", "w4" and "w5". Each vector is of length n and gives the initial values of the corresponding parameter.}
  \item{init2}{an optional list of four numeric values with names "d", "sigma2", "r0" and "b". Each value is the initial value of the corresponding parameter.}  
  \item{proposal}{an optional list of fourteen numeric values with names "d", "r0", "b", "lambda0", "lambda1", "lambda2", "lambda3", "lambda4", "lambda5", "mu0", "mu2", "mu3", "mu4" and "mu5". Each value is the proposal standard deviation for the corresponding parameter.}
  \item{model}{an optional list of four numeric values with names "cut.off" (cut off value for data), "UUB" (upperbound of uniform distributions for r0 and b), "VUB" (upperbound of uniform distribution for sigma2) and "DUB" (upperbound of uniform distribution for d), which are the model setting parameters.}
  \item{running}{an optional list of four numeric values with names "iter" (number of iterations), "thin" (thinning number), "burnin" (burnin number) and "seed" (random seed), which are the parameters to set up MCMC. }
  \item{monitor}{an optional list of three integers with names "obs1", "obs2" and "obs3", which are the indices of the three monitored observations. The default are the three observations with counts closest to the 25\%, 50\% and 75\% percentile of count.1, respectively. In case of multiple matches, the first one in count.1 is used.}
}
\value{
  A list including elements
  \item{result}{a dataframe containg the analysis result. It has seven columns with names "posterior.probability.0", "posterior.probability.1", "posterior.probability.2", "posterior.probability.3", "posterior.probability.4", "posterior.probability.5" and "interaction.category".}
  \item{acceptance}{a list of acceptance rates with names "r0", "b", "d", "lambda0.min", "lambda0.max", "lambda1.min", "lambda1.max", "lambda2.min", "lambda2.max","lambda3.min", "lambda3.max", "lambda4.min", "lambda4.max", "lambda5.min", "lambda5.max","mu0.min", "mu0.max","mu2.min", "mu2.max","mu3.min", "mu3.max","mu4.min", "mu4.max","mu5.min", "mu5.max". The last 22 values give the ranges of acceptance rates for lambda0, lambda1, lambda2, lambda3, lambda4, lambda5, mu0, mu2, mu3, mu4 and mu5.}
  \item{samples}{a list of 55 numeric vectors of stored samples (after thining) with names "d", "sigma2", "r0", "b", 
	"w0_obs1", "w1_obs1", "w2_obs1", "w3_obs1", "w4_obs1", "w5_obs1",
	"lambda0_obs1", "lambda1_obs1", "lambda2_obs1", "lambda3_obs1", "lambda4_obs1", "lambda5_obs1",
	"mu0_obs1", "mu2_obs1", "mu3_obs1", "mu4_obs1", "mu5_obs1", 
	"w0_obs2", "w1_obs2", "w2_obs2", "w3_obs2", "w4_obs2", "w5_obs2",
	"lambda0_obs2", "lambda1_obs2", "lambda2_obs2", "lambda3_obs2", "lambda4_obs2", "lambda5_obs2",
	"mu0_obs2", "mu2_obs2", "mu3_obs2", "mu4_obs2", "mu5_obs2",
	"w0_obs3", "w1_obs3", "w2_obs3", "w3_obs3", "w4_obs3", "w5_obs3",
	"lambda0_obs3", "lambda1_obs3", "lambda2_obs3", "lambda3_obs3", "lambda4_obs3", "lambda5_obs3",
	"mu0_obs3", "mu2_obs3", "mu3_obs3", "mu4_obs3", "mu5_obs3",
	the last 51 vectors give the samples of pi, lambda's and mu's for the three monitored observations.}
  \item{computing.time}{a vaule of computing time, in unit of seconds.}
}
\examples{
data("simdata.OS")
n=length(simdata.OS$count.1)
output=MDOS(count.1=simdata.OS$count.1,count.2=simdata.OS$count.2,
            dirichlet.const=list("c0"=simdata.OS$dirichlet.c0,
                                 "c1"=simdata.OS$dirichlet.c1,
                                 "c2"=simdata.OS$dirichlet.c2,
                                 "c3"=simdata.OS$dirichlet.c3,
                                 "c4"=simdata.OS$dirichlet.c4,
                                 "c5"=simdata.OS$dirichlet.c5),
            init1=list("z"=rep(1,n),
                       "lambda0"=rep(20.0,n),"lambda1"=rep(15.0,n),
                       "lambda2"=rep(10.0,n),"lambda3"=rep(15.0,n),
                       "lambda4"=rep(2.0,n),"lambda5"=rep(3.0,n),
                       "mu0"=rep(10.0,n),"mu2"=rep(20.0,n),
                       "mu3"=rep(4.0,n),"mu4"=rep(10.0,n),
                       "mu5"=rep(2.0,n),
                       "w0"=rep(0.2,n),"w1"=rep(0.1,n),
                       "w2"=rep(0.1,n),"w3"=rep(0.2,n),
                       "w4"=rep(0.1,n),"w5"=rep(0.3,n)),
           init2=list("d"=5000,"sigma2"=35^2,"r0"=6.0,"b"=2.0),
           proposal=list("d"=100.0,"r0"=0.008,"b"=0.025,
                         "lambda0"=1.0,"lambda1"=1.0,"lambda2"=1.0,
                         "lambda3"=1.0,"lambda4"=1.0,"lambda5"=1.0,
                         "mu0"=0.8,"mu2"=0.8,"mu3"=0.8,
                         "mu4"=0.8,"mu5"=0.8))
output$result$interaction.category
}


