\name{MDTS}
\alias{MDTS}
\title{
Second Step of Two-Step Model
}
\description{
The second step of the two-step model for detecting chromatin interactions with different intensities in two samples. 
}
\usage{
MDTS(count.1,count.2,dirichlet.const,
     init1=list("z"=rep(1,length(count.1)),
                "lambda0"=rep(model$cut.off+10.0,length(count.1)),
                "lambda1"=rep(model$cut.off+5.0,length(count.1)),
                "lambda2"=rep(model$cut.off+5.0,length(count.1)),
                "mu0"=rep(model$cut.off+5.0,length(count.1)),
                "mu2"=rep(model$cut.off+10.0,length(count.1)),
                "w0"=rep(1/3,length(count.1)),
                "w1"=rep(1/3,length(count.1)),
                "w2"=rep(1/3,length(count.1))),
     init2=list("d"=0.1,"r1"=1.0,"a"=1.0),
     proposal=list("d"=0.1,"r1"=0.1,"a"=0.1,"lambda0"=0.1,
                   "lambda1"=0.1,"lambda2"=0.1,
                   "mu0"=0.1,"mu2"=0.1),
     model=list("cut.off"=1,"UUB"=1000,"DUB"=100000),
     running=list("iter"=1000,"thin"=10,"burnin"=200,"seed"=2000),
     monitor=list("obs1"=match(min(abs(count.1-quantile(count.1)[2])),
                               abs(count.1-quantile(count.1)[2])),
                  "obs2"=match(min(abs(count.1-quantile(count.1)[3])),
                               abs(count.1-quantile(count.1)[3])),
                  "obs3"=match(min(abs(count.1-quantile(count.1)[4])),
                               abs(count.1-quantile(count.1)[4]))))}
\author{
Liang Niu
}
\arguments{
  \item{count.1}{a vector specifying the observations (counts)of DNA fragment pairs claimed as true pairs (pairs formed by true chromatin interactions)in sample one by MC_DIST model (the model in the first step of the two-step model).}
  \item{count.2}{a vector specifying the observations (counts)of DNA fragment pairs claimed as true pairs (pairs formed by true chromatin interactions)in sample two by MC_DIST model (the model in the first step of the two-step model).}
  \item{dirichlet.const}{a list of three numeric vectors with names "c0", "c1" and "c2". Each vector is of length n (length of count.1 and count.2) and specifies the corresponding constant factors in the dirichlet distributions for the weights of the three components of the model.}
  \item{init1}{an optional list of nine numeric vectors with names "z", "lambda0", "lambda1", "lambda2", "mu0", "mu2", "w0", "w1" and "w2". Each vector is of length n and gives the initial values of the corresponding parameter.}
  \item{init2}{an optional list of three numeric values with names "d", "r1" and "a". Each value is the initial value of the corresponding parameter.}  
  \item{proposal}{an optional list of eight numeric values with names "d", "r1", "a", "lambda0", "lambda1", "lambda2", "mu0" and "mu2". Each value is the proposal standard deviation for the corresponding parameter.}
  \item{model}{an optional list of three numeric values with names "cut.off" (cut off value for data), "UUB" (upperbound of uniform distributions for r1 and a) and "DUB" (upperbound of uniform distribution for d), which are the model setting parameters.}
  \item{running}{an optional list of four numeric values with names "iter" (number of iterations), "thin" (thinning number), "burnin" (burnin number) and "seed" (random seed), which are the parameters to set up MCMC. }
  \item{monitor}{an optional list of three integers with names "obs1", "obs2" and "obs3", which are the indices of the three monitored observations. The default are the three observations with counts closest to the 25\%, 50\% and 75\% percentile of count.1, respectively. In case of multiple matches, the first one in count.1 is used.}
}
\value{
  A list including elements
  \item{result}{a dataframe containg the analysis result. It has four columns with names "posterior.probability.0", "posterior.probability.1", "posterior.probability.2" and "interaction.category".}
  \item{acceptance}{a list of acceptance rates with names "r1", "a", "d", "lambda0.min", "lambda0.max", "lambda1.min", "lambda1.max", "lambda2.min", "lambda2.max","mu0.min", "mu0.max","mu2.min", "mu2.max",. The last ten values give the ranges of acceptance rates for lambda0, lambda1, lambda2, mu0 and mu2.}
  \item{samples}{a list of 27 numeric vectors of stored samples (after thining) with names "d", "r1", "a", "w0_obs1", "w1_obs1", "w2_obs1", "lambda0_obs1", "lambda1_obs1", "lambda2_obs1", "mu0_obs1", "mu2_obs1", "w0_obs2", "w1_obs2", "w2_obs2", "lambda0_obs2", "lambda1_obs2", "lambda2_obs2", "mu0_obs2", "mu2_obs2", "w0_obs3", "w1_obs3", "w2_obs3", "lambda0_obs3", "lambda1_obs3", "lambda2_obs3", "mu0_obs3" and "mu2_obs3", the last 24 vectors give the samples of pi, lambda's and mu's for the three monitored observations.}
  \item{computing.time}{a vaule of computing time, in unit of seconds.}
}
\examples{
data("simdata.TS")
n=length(simdata.TS$count.1)
output=MDTS(count.1=simdata.TS$count.1,count.2=simdata.TS$count.2,
            dirichlet.const=list("c0"=simdata.TS$dirichlet.c0,
                                 "c1"=simdata.TS$dirichlet.c1,
                                 "c2"=simdata.TS$dirichlet.c2),
            init1=list("z"=rep(1,n),"lambda0"=rep(10.0,n),
                       "lambda1"=rep(5.0,n),"lambda2"=rep(5.0,n),
                       "mu0"=rep(5.0,n),"mu2"=rep(10.0,n),
                       "w0"=rep(0.3,n),"w1"=rep(0.4,n),"w2"=rep(0.3,n)),
            init2=list("d"=0.5,"r1"=10.0,"a"=1.0),
            proposal=list("d"=0.031,"r1"=0.01,"a"=0.036,"lambda0"=0.7,
                          "lambda1"=0.7,"lambda2"=0.7,"mu0"=0.7,"mu2"=0.7))
output$result$interaction.category
}


