#define _ISOC99_SOURCE
#include<stdio.h>
#include<stddef.h>
#include<math.h>
#include<time.h>
#include<gsl/gsl_rng.h>
#include<gsl/gsl_randist.h>
#include<gsl/gsl_cdf.h>
#include<gsl/gsl_sf_gamma.h>
#include <R.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_statistics.h>


/* define result stuct */

typedef struct{
double sample;
int accept;
} Result;

/* include functions */
double ran_beta(gsl_rng *r,double a,double b);
double adjust(double a,double b,int sv);
int sample_z_j(gsl_rng *r,double pi_j,double lambda1_j,double lambda0_j,int x_j,int sv);
Result sample_lambda1_j(gsl_rng *r,double lambda1_j,double lambda0_j,int x_j,int z_j,double m,int sv,double proposal_sd);
Result sample_lambda0_j(gsl_rng *r,double lambda0_j,double lambda1_j,int x_j,int z_j,double m,double b,double r0,int sv,double proposal_sd);
double sample_pi_j(gsl_rng *r,int z_j,double d,double mcmdr_j,double mmcmmdr);
Result sample_d(gsl_rng *r,int const DUB,double d,int n,double pi[],double mcmdr[],double mmcmmdr,double proposal_sd);
double sample_m(gsl_rng *r,double const VUB,int n,double sum_lambdadiff_sq);
Result sample_r0(gsl_rng *r,int const UUB,double r0,double b,int n,double sum_lambda0,double proposal_sd);
Result sample_b(gsl_rng *r,int const UUB,double b,double r0,int n,double sum_lambda0,double sum_log_lambda0,double proposal_sd);

/* define sampling functions */
double ran_beta(gsl_rng *r,double a,double b){
	double w=0.0;
	while (w==0.0 || w==1.0){
		w=gsl_ran_beta(r,a,b);
	}
	
	return w;
}

double adjust(double a,double b,int sv){
	double w,log_a_part,log_b_part;
	if (fmax(a,b)<=700){
		log_a_part=a+log(gsl_cdf_poisson_Q(sv-1,a));
		log_b_part=b+log(gsl_cdf_poisson_Q(sv-1,b));
		if (log_a_part!=log_a_part || log_a_part==log(0.0)) log_a_part=sv*log(a)-gsl_sf_lnfact(sv);
		if (log_b_part!=log_b_part || log_b_part==log(0.0)) log_b_part=sv*log(b)-gsl_sf_lnfact(sv);
		w=log_a_part-log_b_part;
	}
	else if (fmin(a,b)>700){
		w=a-b;
	}
	else{
		if (a>b){
			w=a-b-log(gsl_cdf_poisson_Q(sv-1,b));
		}
		else{
			w=a-b+log(gsl_cdf_poisson_Q(sv-1,a));
		}
	}
	return w;
}

int sample_z_j(gsl_rng *r,double pi_j,double lambda1_j,double lambda0_j,int x_j,int sv){
	double w;
	w=exp(log(1/pi_j-1)+x_j*(log(lambda0_j)-log(lambda1_j))+adjust(lambda1_j,lambda0_j,sv));
	return (int) gsl_ran_bernoulli(r,1/(1+w));
}

Result sample_lambda1_j(gsl_rng *r,double lambda1_j,double lambda0_j,int x_j,int z_j,double m,int sv,double proposal_sd){
	double lambda1_j_old=lambda1_j;
	double lambda1_j_new=gsl_ran_lognormal(r,log(lambda1_j),proposal_sd);

	Result rslt;
	double alpha;
	if (lambda1_j_new<=lambda0_j){
		rslt.sample=lambda1_j_old;
		rslt.accept=0;
	}
	else{
		if (z_j==0) alpha=-0.5*(lambda1_j_new+lambda1_j_old-2*lambda0_j)*(lambda1_j_new-lambda1_j_old)/m+log(lambda1_j_new)-log(lambda1_j_old);
		else alpha=-(0.5*(lambda1_j_new+lambda1_j_old-2*lambda0_j)/m)*(lambda1_j_new-lambda1_j_old)+(x_j+1)*(log(lambda1_j_new)-log(lambda1_j_old))+adjust(lambda1_j_old,lambda1_j_new,sv);
	
		if (log(gsl_rng_uniform_pos(r))<alpha){
			rslt.sample=lambda1_j_new;
			rslt.accept=1;
		}
		else{
			rslt.sample=lambda1_j_old;
                	rslt.accept=0;
		}
	}

	return rslt;
}

Result sample_lambda0_j(gsl_rng *r,double lambda0_j,double lambda1_j,int x_j,int z_j,double m,double b,double r0,int sv,double proposal_sd){
        double lambda0_j_old=lambda0_j;
        double lambda0_j_new=gsl_ran_lognormal(r,log(lambda0_j),proposal_sd);

       
	Result rslt;
	double alpha;
        if (lambda0_j_new>=lambda1_j){
                rslt.sample=lambda0_j_old;
                rslt.accept=0;
        }
        else{
        	if (z_j==1) alpha=b*(log(lambda0_j_new)-log(lambda0_j_old))-(lambda0_j_new-lambda0_j_old)*(b/r0-0.5*(2*lambda1_j-lambda0_j_new-lambda0_j_old)/m);
        	else alpha=(b+x_j)*(log(lambda0_j_new)-log(lambda0_j_old))-(lambda0_j_new-lambda0_j_old)*(b/r0-0.5*(2*lambda1_j-lambda0_j_new-lambda0_j_old)/m)+adjust(lambda0_j_old,lambda0_j_new,sv);

        	if (log(gsl_rng_uniform_pos(r))<alpha){
                	rslt.sample=lambda0_j_new;
                	rslt.accept=1;
        	}
        	else{
                	rslt.sample=lambda0_j_old;
                	rslt.accept=0;
        	}
	}

        return rslt;
}

double sample_pi_j(gsl_rng *r,int z_j,double d,double mcmdr_j,double mmcmmdr){
	if (z_j==1) return ran_beta(r,mcmdr_j*d+1,mmcmmdr*d);
	else return ran_beta(r,mcmdr_j*d,mmcmmdr*d+1);
}

Result sample_d(gsl_rng *r,int const DUB,double d,int n,double pi[],double mcmdr[],double mmcmmdr,double proposal_sd){
	double d_old=d;
	double lb=fmax(d-proposal_sd,0),ub=fmin(d+proposal_sd,DUB);
	double d_new=gsl_rng_uniform_pos(r)*(ub-lb)+lb;
	
	Result rslt;
	double alpha=0.0;
	int i;
	for (i=0;i<n;i++){
		alpha=alpha+gsl_sf_lnbeta(mcmdr[i]*d_old,mmcmmdr*d_old)-gsl_sf_lnbeta(mcmdr[i]*d_new,mmcmmdr*d_new)
			+(d_new-d_old)*mcmdr[i]*log(pi[i])+(d_new-d_old)*mmcmmdr*log(1-pi[i]);
	}
	alpha=alpha+log(ub-lb)-log(fmin(d_new+proposal_sd,DUB)-fmax(d_new-proposal_sd,0));
	
        if (log(gsl_rng_uniform_pos(r))<alpha){
                rslt.sample=d_new;
                rslt.accept=1;
        }
        else{
                rslt.sample=d_old;
                rslt.accept=0;
        }

        return rslt;
}

double sample_m(gsl_rng *r,double const VUB,int n,double sum_lambdadiff_sq){
	double w=VUB+1.0;
	while (w>=VUB){
		w=1/gsl_ran_gamma(r,0.5*n-0.5,2/sum_lambdadiff_sq);
	}
	return w;
}

Result sample_r0(gsl_rng *r,int const UUB,double r0,double b,int n,double sum_lambda0,double proposal_sd){
	double r0_old=r0;
	double r0_new=gsl_ran_lognormal(r,log(r0),proposal_sd);

	Result rslt;
	double alpha;
	if (r0_new>=UUB){
		rslt.sample=r0_old;
		rslt.accept=0;
	}
	else {
		alpha=(n*b-1)*(log(r0_old)-log(r0_new))+b*sum_lambda0*(1/r0_old-1/r0_new);
		if (log(gsl_rng_uniform_pos(r))<alpha){
                	rslt.sample=r0_new;
                	rslt.accept=1;
       		}
        	else{
                	rslt.sample=r0_old;
                	rslt.accept=0;
        	}
	}
        return rslt;
}
		
Result sample_b(gsl_rng *r,int const UUB,double b,double r0,int n,double sum_lambda0,double sum_log_lambda0,double proposal_sd){
	double b_old=b;
	double b_new=gsl_ran_lognormal(r,log(b),proposal_sd);
	
	Result rslt;
	double alpha;
        if (b_new>=UUB){
                rslt.sample=b_old;
                rslt.accept=0;
        }
	else{
		alpha=n*(log(b_new)*b_new-log(b_old)*b_old)+(b_new-b_old)*(sum_log_lambda0-sum_lambda0/r0-n*log(r0))
			-n*(gsl_sf_lngamma(b_new)-gsl_sf_lngamma(b_old))+log(b_new)-log(b_old);
		if (log(gsl_rng_uniform_pos(r))<alpha){
                        rslt.sample=b_new;
                        rslt.accept=1;
                }
                else{
                        rslt.sample=b_old;
                        rslt.accept=0;
                }
	}
	return rslt;
}



/* main function */

void MCMC(int *n_obs, int *x, double *wmc,
	int *sv, int const *UUB, int const *DUB, double const *VUB,
	int *iter_all, int *thin, int *burnin, int *r_seed,
	int *initial_z, double *initial_lambda1, double *initial_lambda0, double *initial_pi,
	double *initial_d, double *initial_m, double *initial_r0, double *initial_b,
	double *d_proposal_sd, double *r0_proposal_sd, double *b_proposal_sd, double *lambda1_proposal_sd, double *lambda0_proposal_sd,
	int *obs1, int *obs2, int *obs3,
	double *post_prob, int *bd, double *h_accpt, double *lambda1_accpt, double *lambda0_accpt,
	double *d_samples, double *m_samples, double *r0_samples, double *b_samples, 
	double *pi_samples_obs1, double *lambda1_samples_obs1, double *lambda0_samples_obs1,
	double *pi_samples_obs2, double *lambda1_samples_obs2, double *lambda0_samples_obs2,
	double *pi_samples_obs3, double *lambda1_samples_obs3, double *lambda0_samples_obs3,
	time_t *run_time){

	/* get start time */
	time_t start=time(0);

	/* get data size and set useful variable */
        int n=*n_obs, i;
	double mwmc=0.0;
	
	/* if n=1 then quit */
	/*if (n==1) exit(1); */

	/* get mwmc */
	for (i=0;i<n;i++){
		mwmc=mwmc+wmc[i];
	}

	mwmc=mwmc/n;

	
	/* define variables for updating and set initial values */
	int *z_post=(int *)malloc(n*sizeof(int));
	double *lambda1_post=(double *)malloc(n*sizeof(double)),*lambda0_post=(double *)malloc(n*sizeof(double)),*pi_post=(double *)malloc(n*sizeof(double));
        for (i=0;i<n;i++){
		z_post[i]=initial_z[i];
		lambda1_post[i]=initial_lambda1[i];
		lambda0_post[i]=initial_lambda0[i];
		pi_post[i]=initial_pi[i];
        }


	double d_post=*initial_d,m_post=*initial_m,r0_post=*initial_r0,b_post=*initial_b;


	/* define storage variables and set proposal sd's */
	double accept_d=0.0,accept_r0=0.0,accept_b=0.0; 
	
	double *accept_lambda1=(double *)malloc(n*sizeof(double)),*accept_lambda0=(double *)malloc(n*sizeof(double));
	for (i=0;i<n;i++){
		accept_lambda1[i]=0.0;
		accept_lambda0[i]=0.0;
	}
	
	
	/* define additional variables */
	int j,k=0;
	Result tmp;
	double sum_lambda0,sum_log_lambda0,sum_lambdadiff_sq;
	double *sum_post_prob_z=(double *)malloc(n*sizeof(double));
	for (i=0;i<n;i++){
		sum_post_prob_z[i]=0.0;
	}


	/* start Gibbs sampler */
	/* set random generator and seed */
	gsl_rng *r;
	r=gsl_rng_alloc(gsl_rng_mt19937);
	gsl_rng_set(r,*r_seed);	


	/* update parameters */
	for (i=0;i<*iter_all;i++){
		sum_lambda0=0.0; sum_log_lambda0=0.0; sum_lambdadiff_sq=0.0;
		
		for (j=0;j<n;j++){
			z_post[j]=sample_z_j(r,pi_post[j],lambda1_post[j],lambda0_post[j],x[j],*sv);
			
			pi_post[j]=sample_pi_j(r,z_post[j],d_post,wmc[j],mwmc);			
			
			tmp=sample_lambda1_j(r,lambda1_post[j],lambda0_post[j],x[j],z_post[j],m_post,*sv,*lambda1_proposal_sd);
			lambda1_post[j]=tmp.sample;
			accept_lambda1[j]=accept_lambda1[j]+tmp.accept;
			
			tmp=sample_lambda0_j(r,lambda0_post[j],lambda1_post[j],x[j],z_post[j],m_post,b_post,r0_post,*sv,*lambda0_proposal_sd);
			lambda0_post[j]=tmp.sample;
                        accept_lambda0[j]=accept_lambda0[j]+tmp.accept;

			sum_lambdadiff_sq=sum_lambdadiff_sq+pow(lambda1_post[j]-lambda0_post[j],2.0);
			sum_lambda0=sum_lambda0+lambda0_post[j];
                        sum_log_lambda0=sum_log_lambda0+log(lambda0_post[j]);		

			if ((fmod(i+1,*thin)==0) && (i>(*burnin-1))){
				sum_post_prob_z[j]=sum_post_prob_z[j]+1/(1+exp(log(1/pi_post[j]-1)+x[j]*(log(lambda0_post[j])-log(lambda1_post[j]))+adjust(lambda1_post[j],lambda0_post[j],*sv)));
			}
		}
	
			
		tmp=sample_d(r,*DUB,d_post,n,pi_post,wmc,mwmc,*d_proposal_sd);
		d_post=tmp.sample;
		accept_d=accept_d+tmp.accept;

		m_post=sample_m(r,*VUB,n,sum_lambdadiff_sq);
		
		tmp=sample_r0(r,*UUB,r0_post,b_post,n,sum_lambda0,*r0_proposal_sd);
                r0_post=tmp.sample;
                accept_r0=accept_r0+tmp.accept; 	
		
		tmp=sample_b(r,*UUB,b_post,r0_post,n,sum_lambda0,sum_log_lambda0,*b_proposal_sd);
                b_post=tmp.sample;
                accept_b=accept_b+tmp.accept;

		
		if (fmod(i+1,*thin)==0){
			d_samples[k]=d_post;
			m_samples[k]=m_post;
			r0_samples[k]=r0_post;
			b_samples[k]=b_post;
			pi_samples_obs1[k]=pi_post[*obs1-1];
			lambda1_samples_obs1[k]=lambda1_post[*obs1-1];
			lambda0_samples_obs1[k]=lambda0_post[*obs1-1];
			pi_samples_obs2[k]=pi_post[*obs2-1];
			lambda1_samples_obs2[k]=lambda1_post[*obs2-1];
			lambda0_samples_obs2[k]=lambda0_post[*obs2-1];
			pi_samples_obs3[k]=pi_post[*obs3-1];
			lambda1_samples_obs3[k]=lambda1_post[*obs3-1];
			lambda0_samples_obs3[k]=lambda0_post[*obs3-1];
			k=k+1;
		}

	}


	/* free the random number generator */
	gsl_rng_free(r);
	
	
	/* ouput the acceptance rates for r0, b and d */
	/* output bayesian decision*/
	for (j=0;j<n;j++){
                post_prob[j]=sum_post_prob_z[j]/((*iter_all-*burnin)/(*thin));
                if (post_prob[j]>0.5) bd[j]=1;
                else bd[j]=0;
        }
	
	/* ouput the acceptance rates for r0, b and d */
        h_accpt[0]=accept_r0/(*iter_all); h_accpt[1]=accept_b/(*iter_all); h_accpt[2]=accept_d/(*iter_all);

	/* output the acceptance rates for lambda1's and lambda0's */
	double lambda1_accpt_lb=accept_lambda1[0],lambda1_accpt_ub=accept_lambda1[0],lambda0_accpt_lb=accept_lambda0[0],lambda0_accpt_ub=accept_lambda0[0];
	for (j=0;j<(n-1);j++){
		lambda1_accpt_lb=fmin(lambda1_accpt_lb,accept_lambda1[j+1]); lambda1_accpt_ub=fmax(lambda1_accpt_ub,accept_lambda1[j+1]);
		lambda0_accpt_lb=fmin(lambda0_accpt_lb,accept_lambda0[j+1]); lambda0_accpt_ub=fmax(lambda0_accpt_ub,accept_lambda0[j+1]);
		
	}

	lambda1_accpt[0]=lambda1_accpt_lb/(*iter_all); lambda1_accpt[1]=lambda1_accpt_ub/(*iter_all);
	lambda0_accpt[0]=lambda0_accpt_lb/(*iter_all); lambda0_accpt[1]=lambda0_accpt_ub/(*iter_all);

	/* free pointers*/
	free(z_post); free(lambda1_post); free(lambda0_post); free(pi_post); free(accept_lambda1); free(accept_lambda0); free(sum_post_prob_z);

	/* get runing time */
	time_t end=time(0);
	run_time[0]=(end-start);
}



/* include functions */
double **allocate2dDoubleArray(int line,int column);
void deallocate2dDoubleArray(double **array, int line);
void ran_dirichlet_TS(gsl_rng *r,double c[],double p[]);
double adjust_TS(double a,double b,int sv);
int ran_discrete_TS(gsl_rng *r,double w[]);
int sample_z_j_TS(gsl_rng *r,double p_j[],double lambda0_j,double lambda1_j,double lambda2_j,double mu0_j,double mu2_j,int x_j,int y_j,int sv);
void sample_p_j_TS(gsl_rng *r,int z_j,double p_j[3],double d,double w_j[3]);
Result sample_d_TS(gsl_rng *r,int const DUB,double d,int n,double **p,double **w,double proposal_sd);
Result accept_mean_j_TS(gsl_rng *r,double mean_j_new,double mean_j_old,int count_j,int z_j,int flag,double gamma_sp,double gamma_mp,int sv);
Result sample_mean_j_TS(gsl_rng *r,double mean_j,double reference_j,int x_j,int y_j,int z_j,int group,char treat,double gamma_sp,double gamma_mp,int sv,double proposal_sd);
Result sample_mp_TS(gsl_rng *r,int const UUB,double mp,double sp,int n,double sum_parm,double proposal_sd);
Result sample_sp_TS(gsl_rng *r,int const UUB,double sp,double mp,int n,double sum_parm,double sum_log_parm,double proposal_sd);
int unique_max_TS(double m,double v0,double v1,double v2);

/* define sampling functions */

double **allocate2dDoubleArray(int line,int column){
        double **DoubleArray=(double **)malloc(line*sizeof(double *));
        for (int i=0;i<line;i++){
                DoubleArray[i]=(double *)malloc(column*sizeof(double));
        }
        return DoubleArray;
}

void deallocate2dDoubleArray(double **array, int line){
        for(int i=0;i<line;i++) {
                free(array[i]);
        }
        free(array);
}

void ran_dirichlet_TS(gsl_rng *r,double c[],double p[]){
	int i;
	double w[3]={1.0,0.0,0.0};
	while (w[0]==0.0 || w[1]==0.0 || w[2]==0.0){
		gsl_ran_dirichlet(r,3,c,w);
	}

	for (i=0;i<3;i++){
		p[i]=w[i];
	}
}


double adjust_TS(double a,double b,int sv){
	double w,log_a_part,log_b_part;
	if (fmax(a,b)<=700){
		log_a_part=a+log(gsl_cdf_poisson_Q(sv-1,a));
		log_b_part=b+log(gsl_cdf_poisson_Q(sv-1,b));
		if (log_a_part!=log_a_part || log_a_part==log(0.0)) log_a_part=sv*log(a)-gsl_sf_lnfact(sv);
		if (log_b_part!=log_b_part || log_b_part==log(0.0)) log_b_part=sv*log(b)-gsl_sf_lnfact(sv);
		w=log_a_part-log_b_part;
	}
	else if (fmin(a,b)>700){
		w=a-b;
	}
	else{
		if (a>b){
			w=a-b-log(gsl_cdf_poisson_Q(sv-1,b));
		}
		else{
			w=a-b+log(gsl_cdf_poisson_Q(sv-1,a));
		}
	}
	return w;
}


int ran_discrete_TS(gsl_rng *r,double w[]){
	double sum_w=w[0]+w[1]+w[2];
	double a=gsl_rng_uniform_pos(r)*sum_w;
	if (a<w[0]) return 0;
	else if (a<(w[0]+w[1])) return 1;
	else return 2;
}

int sample_z_j_TS(gsl_rng *r,double p_j[],double lambda0_j,double lambda1_j,double lambda2_j,double mu0_j,double mu2_j,int x_j,int y_j,int sv){
	double w[3],max_w;
	w[0]=log(p_j[0]);
	w[1]=log(p_j[1])+x_j*(log(lambda1_j)-log(lambda0_j))+y_j*(log(lambda1_j)-log(mu0_j))+adjust_TS(lambda0_j,lambda1_j,sv)+adjust_TS(mu0_j,lambda1_j,sv);
	w[2]=log(p_j[2])+x_j*(log(lambda2_j)-log(lambda0_j))+y_j*(log(mu2_j)-log(mu0_j))+adjust_TS(lambda0_j,lambda2_j,sv)+adjust_TS(mu0_j,mu2_j,sv);
	max_w=fmax(fmax(w[0],w[1]),w[2]);
	w[0]=exp(w[0]-max_w); w[1]=exp(w[1]-max_w); w[2]=exp(w[2]-max_w);
	return  ran_discrete_TS(r,w);
}

void sample_p_j_TS(gsl_rng *r,int z_j,double p_j[3],double d,double w_j[3]){
        double c[3]={w_j[0]*d,w_j[1]*d,w_j[2]*d};
	c[z_j]=c[z_j]+1.0;
        ran_dirichlet_TS(r,c,p_j);
}

Result sample_d_TS(gsl_rng *r,int const DUB,double d,int n,double **p,double **w,double proposal_sd){
	double d_old=d;
	double lb=fmax(d-proposal_sd,0),ub=fmin(d+proposal_sd,DUB);
	double d_new=gsl_rng_uniform_pos(r)*(ub-lb)+lb;

	double sum_w;

	double alpha=0.0;
	int i;
	for (i=0;i<n;i++){
		sum_w=w[i][0]+w[i][1]+w[i][2];
		alpha=alpha+gsl_sf_lngamma(w[i][0]*d_old)-gsl_sf_lngamma(w[i][0]*d_new)
			+gsl_sf_lngamma(w[i][1]*d_old)-gsl_sf_lngamma(w[i][1]*d_new)
			+gsl_sf_lngamma(w[i][2]*d_old)-gsl_sf_lngamma(w[i][2]*d_new)
			+gsl_sf_lngamma(sum_w*d_new)-gsl_sf_lngamma(sum_w*d_old)	
			+(d_new-d_old)*(w[i][0]*log(p[i][0])+w[i][1]*log(p[i][1])+w[i][2]*log(p[i][2]));
	}
	alpha=alpha+log(ub-lb)-log(fmin(d_new+proposal_sd,DUB)-fmax(d_new-proposal_sd,0));

	Result rslt;
        if (log(gsl_rng_uniform_pos(r))<alpha){
                rslt.sample=d_new;
                rslt.accept=1;
        }
        else{
                rslt.sample=d_old;
                rslt.accept=0;
        }

        return rslt;
}

Result accept_mean_j_TS(gsl_rng *r,double mean_j_new,double mean_j_old,int count_j,int z_j,int flag,double gamma_sp,double gamma_mp,int sv){
	Result rslt;	

	double alpha;
	if (z_j==flag){
		if (flag==1){
			alpha=(gamma_sp+count_j)*(log(mean_j_new)-log(mean_j_old))
				-(gamma_sp/gamma_mp)*(mean_j_new-mean_j_old)+2*adjust_TS(mean_j_old,mean_j_new,sv);
		}
		else{
			alpha=(gamma_sp+count_j)*(log(mean_j_new)-log(mean_j_old))
                                -(gamma_sp/gamma_mp)*(mean_j_new-mean_j_old)+adjust_TS(mean_j_old,mean_j_new,sv);
		}
	}
	else alpha=gamma_sp*(log(mean_j_new)-log(mean_j_old))-(gamma_sp/gamma_mp)*(mean_j_new-mean_j_old);	

	if (log(gsl_rng_uniform_pos(r))<alpha){
		rslt.sample=mean_j_new;
		rslt.accept=1;
	}
	else{
		rslt.sample=mean_j_old;
                rslt.accept=0;
	}

	return rslt;
}

Result sample_mean_j_TS(gsl_rng *r,double mean_j,double reference_j,int x_j,int y_j,int z_j,int group,char treat,double gamma_sp,double gamma_mp,int sv,double proposal_sd){
        double mean_j_old=mean_j;
        double mean_j_new=gsl_ran_lognormal(r,log(mean_j),proposal_sd);

	Result rslt;
	if (group==0){
		if (treat=='x'){
			if (mean_j_new<=reference_j){
				rslt.sample=mean_j_old;
		                rslt.accept=0;
			}
			else rslt=accept_mean_j_TS(r,mean_j_new,mean_j_old,x_j,z_j,0,gamma_sp,gamma_mp,sv);
		}
		else{
			if (mean_j_new>=reference_j){
				rslt.sample=mean_j_old;
                                rslt.accept=0;
                        }
			else rslt=accept_mean_j_TS(r,mean_j_new,mean_j_old,y_j,z_j,0,gamma_sp,gamma_mp,sv);
		}
	}
	else if (group==1){
		rslt=accept_mean_j_TS(r,mean_j_new,mean_j_old,x_j+y_j,z_j,1,gamma_sp,gamma_mp,sv);
	}
	else{
		if (treat=='x'){
                	if (mean_j_new>=reference_j){
                                rslt.sample=mean_j_old;
                                rslt.accept=0;
                        }
                        else rslt=accept_mean_j_TS(r,mean_j_new,mean_j_old,x_j,z_j,2,gamma_sp,gamma_mp,sv);
                }
                else{
                        if (mean_j_new<=reference_j){
                                rslt.sample=mean_j_old;
                                rslt.accept=0;
                        }
                        else rslt=accept_mean_j_TS(r,mean_j_new,mean_j_old,y_j,z_j,2,gamma_sp,gamma_mp,sv);
                }
	}	
		
	return rslt;
}

Result sample_mp_TS(gsl_rng *r,int const UUB,double mp,double sp,int n,double sum_parm,double proposal_sd){
	double mp_old=mp;
	double mp_new=gsl_ran_lognormal(r,log(mp),proposal_sd);
	

	Result rslt;
	if (mp_new>=UUB){
		rslt.sample=mp_old;
		rslt.accept=0;
	}
	else {
		double alpha;
		alpha=(n*sp-1)*(log(mp_old)-log(mp_new))+sp*sum_parm*(1/mp_old-1/mp_new);
		if (log(gsl_rng_uniform_pos(r))<alpha){
                	rslt.sample=mp_new;
                	rslt.accept=1;
       		}
        	else{
                	rslt.sample=mp_old;
                	rslt.accept=0;
        	}
	}
        return rslt;
}
		
Result sample_sp_TS(gsl_rng *r,int const UUB,double sp,double mp,int n,double sum_parm,double sum_log_parm,double proposal_sd){
	double sp_old=sp;
	double sp_new=gsl_ran_lognormal(r,log(sp),proposal_sd);
	
	Result rslt;
        if (sp_new>=UUB){
                rslt.sample=sp_old;
                rslt.accept=0;
        }
	else{
		double alpha;
		alpha=n*(log(sp_new)*sp_new-log(sp_old)*sp_old)+(sp_new-sp_old)*(sum_log_parm-sum_parm/mp-n*log(mp))
			-n*(gsl_sf_lngamma(sp_new)-gsl_sf_lngamma(sp_old))+log(sp_new)-log(sp_old);
		if (log(gsl_rng_uniform_pos(r))<alpha){
                        rslt.sample=sp_new;
                        rslt.accept=1;
                }
                else{
                        rslt.sample=sp_old;
                        rslt.accept=0;
                }
	}
	return rslt;
}


/* find number of maximum values among 3 double values */
int unique_max_TS(double m,double v0,double v1,double v2){
	double v[3]={v0,v1,v2};
	int i,k=0;
	
	for (i=0;i<3;i++){
		if (m==v[i]) k++;
	}
	
	return k;
}


/* main function */

void MDTS(int *n_obs,int *x,int *y,double *w0,double *w1,double *w2,
	int *sv,int const *UUB,int const *DUB,
	int *iter_all,int *thin,int *burnin,int *r_seed,
	int *initial_z,double *initial_lambda0,double *initial_lambda1,double *initial_lambda2,double *initial_mu0,double *initial_mu2,
	double *initial_p0,double *initial_p1,double *initial_p2,
	double *initial_h,
	double *proposal_sd,
	int *obs,
	double *post_prob_z_0,double *post_prob_z_1,double *post_prob_z_2,int *bd,
	double *h_accpt,double *lambda0_accpt,double *lambda1_accpt,double *lambda2_accpt,double *mu0_accpt,double *mu2_accpt,
	double *d_samples,double *r1_samples, double *a_samples,
	double *p0_samples_1,double *p1_samples_1,double *p2_samples_1,
	double *lambda0_samples_1,double *lambda1_samples_1,double *lambda2_samples_1,double *mu0_samples_1,double *mu2_samples_1,
	double *p0_samples_2,double *p1_samples_2,double *p2_samples_2,
	double *lambda0_samples_2,double *lambda1_samples_2,double *lambda2_samples_2,double *mu0_samples_2,double *mu2_samples_2,
	double *p0_samples_3,double *p1_samples_3,double *p2_samples_3,
	double *lambda0_samples_3,double *lambda1_samples_3,double *lambda2_samples_3,double *mu0_samples_3,double *mu2_samples_3,
	time_t *run_time){

	/* get start time */
	time_t start=time(0);

	/* find datasize */
        int n=*n_obs,i;
	
	/* read in weight information */
	double **w=allocate2dDoubleArray(n,3);

	for (i=0;i<n;i++){
		w[i][0]=w0[i]; w[i][1]=w1[i]; w[i][2]=w2[i];
	}

	
	/* define variables for updating and set initial values */
	int *z_post=(int *)malloc(n*sizeof(int));
	double *lambda0_post=(double *)malloc(n*sizeof(double)),*lambda1_post=(double *)malloc(n*sizeof(double)),*lambda2_post=(double *)malloc(n*sizeof(double));
	double *mu0_post=(double *)malloc(n*sizeof(double)),*mu2_post=(double *)malloc(n*sizeof(double));
	double **p_post=allocate2dDoubleArray(n,3);
	for (i=0;i<n;i++){
		z_post[i]=initial_z[i];
		lambda0_post[i]=initial_lambda0[i];
		lambda1_post[i]=initial_lambda1[i];
		lambda2_post[i]=initial_lambda2[i];
		mu0_post[i]=initial_mu0[i];
		mu2_post[i]=initial_mu2[i];
		p_post[i][0]=initial_p0[i];
		p_post[i][1]=initial_p1[i];
		p_post[i][2]=initial_p2[i];
	}
	
	double d_post=initial_h[0],r1_post=initial_h[1],a_post=initial_h[2];
	
	/* define storage variables and set proposal sd's */

	double accept_d=0.0,accept_r1=0.0,accept_a=0.0; 
	
	double *accept_lambda0=(double *)malloc(n*sizeof(double)),*accept_lambda1=(double *)malloc(n*sizeof(double)),*accept_lambda2=(double *)malloc(n*sizeof(double));
	double *accept_mu0=(double *)malloc(n*sizeof(double)),*accept_mu2=(double *)malloc(n*sizeof(double));
	for (i=0;i<n;i++){
		accept_lambda0[i]=0.0;
		accept_lambda1[i]=0.0;
		accept_lambda2[i]=0.0;
		accept_mu0[i]=0.0;
                accept_mu2[i]=0.0;
	}
	
	/* define additional variables */
	Result tmp;
	int j,k=0;
	double sum_parm,sum_log_parm,wt0,wt1,wt2,max_w;
	double *sum_post_prob_z_0=(double *)malloc(n*sizeof(double)),*sum_post_prob_z_1=(double *)malloc(n*sizeof(double));
	for (i=0;i<n;i++){
		sum_post_prob_z_0[i]=0.0;
		sum_post_prob_z_1[i]=0.0;
	}


	/* start Gibbs sampler */
	/* set random generator and seed */
	gsl_rng *r;
	r=gsl_rng_alloc(gsl_rng_mt19937);
	gsl_rng_set(r,*r_seed);	

	/* update parameters */
	for (i=0;i<*iter_all;i++){
		sum_parm=0.0; sum_log_parm=0.0;
		
		for (j=0;j<n;j++){
			z_post[j]=sample_z_j_TS(r,p_post[j],lambda0_post[j],lambda1_post[j],lambda2_post[j],mu0_post[j],mu2_post[j],x[j],y[j],*sv);

			sample_p_j_TS(r,z_post[j],p_post[j],d_post,w[j]);
			
			tmp=sample_mean_j_TS(r,lambda0_post[j],mu0_post[j],x[j],y[j],z_post[j],0,'x',a_post,r1_post,*sv,proposal_sd[3]);
			lambda0_post[j]=tmp.sample;
			accept_lambda0[j]=accept_lambda0[j]+tmp.accept;
			
			tmp=sample_mean_j_TS(r,lambda1_post[j],lambda1_post[j],x[j],y[j],z_post[j],1,'x',a_post,r1_post,*sv,proposal_sd[4]);
			lambda1_post[j]=tmp.sample;
                        accept_lambda1[j]=accept_lambda1[j]+tmp.accept;
			
			tmp=sample_mean_j_TS(r,lambda2_post[j],mu2_post[j],x[j],y[j],z_post[j],2,'x',a_post,r1_post,*sv,proposal_sd[5]);
                        lambda2_post[j]=tmp.sample;
                        accept_lambda2[j]=accept_lambda2[j]+tmp.accept;
		
			tmp=sample_mean_j_TS(r,mu0_post[j],lambda0_post[j],x[j],y[j],z_post[j],0,'y',a_post,r1_post,*sv,proposal_sd[6]);
                        mu0_post[j]=tmp.sample;
                        accept_mu0[j]=accept_mu0[j]+tmp.accept;

			tmp=sample_mean_j_TS(r,mu2_post[j],lambda2_post[j],x[j],y[j],z_post[j],2,'y',a_post,r1_post,*sv,proposal_sd[7]);
                        mu2_post[j]=tmp.sample;
                        accept_mu2[j]=accept_mu2[j]+tmp.accept;


			sum_parm=sum_parm+lambda0_post[j]+lambda1_post[j]+lambda2_post[j]+mu0_post[j]+mu2_post[j];
			sum_log_parm=sum_log_parm+log(lambda0_post[j])+log(lambda1_post[j])+log(lambda2_post[j])+log(mu0_post[j])+log(mu2_post[j]);

			if ((fmod(i+1,*thin)==0) && (i>(*burnin-1))){
				wt0=log(p_post[j][0]);
				wt1=log(p_post[j][1])+x[j]*(log(lambda1_post[j])-log(lambda0_post[j]))+y[j]*(log(lambda1_post[j])-log(mu0_post[j]))
					+adjust_TS(lambda0_post[j],lambda1_post[j],*sv)+adjust_TS(mu0_post[j],lambda1_post[j],*sv);
				wt2=log(p_post[j][2])+x[j]*(log(lambda2_post[j])-log(lambda0_post[j]))+y[j]*(log(mu2_post[j])-log(mu0_post[j]))
                                        +adjust_TS(lambda0_post[j],lambda2_post[j],*sv)+adjust_TS(mu0_post[j],mu2_post[j],*sv);
				max_w=fmax(fmax(wt0,wt1),wt2);
				wt0=exp(wt0-max_w); wt1=exp(wt1-max_w); wt2=exp(wt2-max_w);
                                sum_post_prob_z_0[j]=sum_post_prob_z_0[j]+wt0/(wt0+wt1+wt2);
				sum_post_prob_z_1[j]=sum_post_prob_z_1[j]+wt1/(wt0+wt1+wt2);
			}
		}
				
		tmp=sample_d_TS(r,*DUB,d_post,n,p_post,w,proposal_sd[0]);
		d_post=tmp.sample;
		accept_d=accept_d+tmp.accept;
		
		tmp=sample_mp_TS(r,*UUB,r1_post,a_post,5*n,sum_parm,proposal_sd[1]);
                r1_post=tmp.sample;
                accept_r1=accept_r1+tmp.accept;

		tmp=sample_sp_TS(r,*UUB,a_post,r1_post,5*n,sum_parm,sum_log_parm,proposal_sd[2]);
                a_post=tmp.sample;
                accept_a=accept_a+tmp.accept;

		
		if (fmod(i+1,*thin)==0){
			d_samples[k]=d_post;
			r1_samples[k]=r1_post;
			a_samples[k]=a_post;

			p0_samples_1[k]=p_post[obs[0]-1][0];
			p1_samples_1[k]=p_post[obs[0]-1][1];
			p2_samples_1[k]=p_post[obs[0]-1][2];
			lambda0_samples_1[k]=lambda0_post[obs[0]-1];
			lambda1_samples_1[k]=lambda1_post[obs[0]-1];
			lambda2_samples_1[k]=lambda2_post[obs[0]-1];
			mu0_samples_1[k]=mu0_post[obs[0]-1];
			mu2_samples_1[k]=mu2_post[obs[0]-1];

			p0_samples_2[k]=p_post[obs[1]-1][0];
			p1_samples_2[k]=p_post[obs[1]-1][1];
			p2_samples_2[k]=p_post[obs[1]-1][2];
			lambda0_samples_2[k]=lambda0_post[obs[1]-1];
			lambda1_samples_2[k]=lambda1_post[obs[1]-1];
			lambda2_samples_2[k]=lambda2_post[obs[1]-1];
			mu0_samples_2[k]=mu0_post[obs[1]-1];
			mu2_samples_2[k]=mu2_post[obs[1]-1];

			p0_samples_3[k]=p_post[obs[2]-1][0];
			p1_samples_3[k]=p_post[obs[2]-1][1];
			p2_samples_3[k]=p_post[obs[2]-1][2];
			lambda0_samples_3[k]=lambda0_post[obs[2]-1];
			lambda1_samples_3[k]=lambda1_post[obs[2]-1];
			lambda2_samples_3[k]=lambda2_post[obs[2]-1];
			mu0_samples_3[k]=mu0_post[obs[2]-1];
			mu2_samples_3[k]=mu2_post[obs[2]-1];

			k=k+1;
		}
		
	}
	
	/* free the random number generator */
	gsl_rng_free(r);

	
	/* find posterior probabilities for z_j=0, 1 or 2, and make bayesian decision*/
	double max_p;

	for (j=0;j<n;j++){
		post_prob_z_0[j]=sum_post_prob_z_0[j]/((*iter_all-*burnin)/(*thin));
		post_prob_z_1[j]=sum_post_prob_z_1[j]/((*iter_all-*burnin)/(*thin));
		post_prob_z_2[j]=fmax(0.0,1.0-post_prob_z_0[j]-post_prob_z_1[j]);

		max_p=fmax(fmax(post_prob_z_0[j],post_prob_z_1[j]),post_prob_z_2[j]);
		if (unique_max_TS(max_p,post_prob_z_0[j],post_prob_z_1[j],post_prob_z_2[j])>1){
			bd[j]=-1;
		}
		else{
			if (post_prob_z_0[j]==max_p) bd[j]=0;
			else if (post_prob_z_1[j]==max_p) bd[j]=1;
			else bd[j]=2;
		}
        }


	/* output the acceptance rates for r1 and a */

        h_accpt[0]=accept_r1/(*iter_all); h_accpt[1]=accept_a/(*iter_all); h_accpt[2]=accept_d/(*iter_all);

	/* output the acceptance rates for lambda's and mu's */
	double lambda0_accpt_lb=accept_lambda0[0],lambda0_accpt_ub=accept_lambda0[0],lambda1_accpt_lb=accept_lambda1[0],lambda1_accpt_ub=accept_lambda1[0],lambda2_accpt_lb=accept_lambda2[0],lambda2_accpt_ub=accept_lambda2[0];
	double mu0_accpt_lb=accept_mu0[0],mu0_accpt_ub=accept_mu0[0],mu2_accpt_lb=accept_mu2[0],mu2_accpt_ub=accept_mu2[0];

	for (j=0;j<(n-1);j++){
		lambda0_accpt_lb=fmin(lambda0_accpt_lb,accept_lambda0[j+1]); lambda0_accpt_ub=fmax(lambda0_accpt_ub,accept_lambda0[j+1]);
		lambda1_accpt_lb=fmin(lambda1_accpt_lb,accept_lambda1[j+1]); lambda1_accpt_ub=fmax(lambda1_accpt_ub,accept_lambda1[j+1]);
		lambda2_accpt_lb=fmin(lambda2_accpt_lb,accept_lambda2[j+1]); lambda2_accpt_ub=fmax(lambda2_accpt_ub,accept_lambda2[j+1]);
		mu0_accpt_lb=fmin(mu0_accpt_lb,accept_mu0[j+1]); mu0_accpt_ub=fmax(mu0_accpt_ub,accept_mu0[j+1]);
		mu2_accpt_lb=fmin(mu2_accpt_lb,accept_mu2[j+1]); mu2_accpt_ub=fmax(mu2_accpt_ub,accept_mu2[j+1]);
	}

	lambda0_accpt[0]=lambda0_accpt_lb/(*iter_all); lambda0_accpt[1]=lambda0_accpt_ub/(*iter_all);
	lambda1_accpt[0]=lambda1_accpt_lb/(*iter_all); lambda1_accpt[1]=lambda1_accpt_ub/(*iter_all);
	lambda2_accpt[0]=lambda2_accpt_lb/(*iter_all); lambda2_accpt[1]=lambda2_accpt_ub/(*iter_all);
	mu0_accpt[0]=mu0_accpt_lb/(*iter_all); mu0_accpt[1]=mu0_accpt_ub/(*iter_all);
	mu2_accpt[0]=mu2_accpt_lb/(*iter_all); mu2_accpt[1]=mu2_accpt_ub/(*iter_all);

	/* free the pointers */
	deallocate2dDoubleArray(w,n); deallocate2dDoubleArray(p_post,n);
	free(z_post);
	free(lambda0_post); free(lambda1_post); free(lambda2_post); free(mu0_post); free(mu2_post);
	free(accept_lambda0); free(accept_lambda1); free(accept_lambda2); free(accept_mu0); free(accept_mu2);
	free(sum_post_prob_z_0); free(sum_post_prob_z_1);

	/* get runing time */
	time_t end=time(0);
	run_time[0]=(end-start);
}


/* include functions */

void ran_dirichlet_OS(gsl_rng *r,double c[],double p[]);
double pse_k_OS(double x,int k);
double adjust_OS(double a1,double a2,double b1,double b2,int sv);
int ran_discrete_OS(gsl_rng *r,double w[]);
int sample_z_j_OS(gsl_rng *r,double p_j[],double lambda0_j,double lambda1_j,double lambda2_j,double lambda3_j,double lambda4_j,double lambda5_j,double mu0_j,double mu2_j,double mu3_j,double mu4_j,double mu5_j,int x_j,int y_j,int sv);
void sample_p_j_OS(gsl_rng *r,int z_j,double p_j[6],double d,double w_j[6]);
Result sample_d_OS(gsl_rng *r,double d,int n,double **p,double **w,double proposal_sd);
Result TL_accept_mean_j_OS(gsl_rng *r,double mean_j_new,double mean_j_old,int count_j,int z_j,int flag,double reference_j,double lc_j,double m,int sv);
Result sample_TL_mean_j_OS(gsl_rng *r,double mean_j,double reference_j,double lc_j,int x_j,int y_j,int z_j,int group,char treat,double m,int sv,double proposal_sd);
Result FL_accept_mean_j_OS(gsl_rng *r,double mean_j_new,double mean_j_old,int count_j,int z_j,int flag,double reference_j,double b,double r0,double ajst_j,int sv);
Result sample_FL_mean_j_OS(gsl_rng *r,double mean_j,double mean1_j,double mean2_j,double mean3_j,double lambda0_j,double mu0_j,double lambda1_j,double lambda2_j,double mu2_j,double lambda3_j,double mu4_j,int x_j,int y_j,int z_j,int group,char treat,double m,double b,double r0,int sv,double proposal_sd);
double sample_m_OS(gsl_rng *r,int const VUB,int n,double sum_lambdadiff_sq);
Result sample_r0_OS(gsl_rng *r,int const UUB,double r0,double b,int n,double sum_parm,double proposal_sd);
Result sample_b_OS(gsl_rng *r,int const UUB,double b,double r0,int n,double sum_parm,double sum_log_parm,double proposal_sd);
int unique_max_OS(double m,double v0,double v1,double v2,double v3,double v4,double v5);

/* define sampling functions */

void ran_dirichlet_OS(gsl_rng *r,double c[],double p[]){
	int i;
	double w[6]={1.0,0.0,0.0,0.0,0.0,0.0};
	while (w[0]==0.0 || w[1]==0.0 || w[2]==0.0 || w[3]==0.0 || w[4]==0.0 || w[5]==0.0){
		gsl_ran_dirichlet(r,6,c,w);
	}
	

	for (i=0;i<6;i++){
		p[i]=w[i];
	}
}


double pse_k_OS(double x,int k){
	int i;
	double w=0.0;
	for (i=0;i<k;i++){
		w=w+pow(x,i*1.0)/gsl_sf_fact(i);
	}
	
	return w;
}


double adjust_OS(double a1,double a2,double b1,double b2,int sv){
	double w,log_a_part,log_b_part;
	if (fmax(fmax(a1,a2),fmax(b1,b2))<=700){
		log_a_part=a1+a2+log(1-gsl_cdf_poisson_P(sv-1,a1)*gsl_cdf_poisson_P(sv-1,a2));
		log_b_part=b1+b2+log(1-gsl_cdf_poisson_P(sv-1,b1)*gsl_cdf_poisson_P(sv-1,b2));
		if (log_a_part!=log_a_part || log_a_part==log(0.0)) log_a_part=log(pse_k_OS(a1,sv)*pow(a2,sv*1.0)+pse_k_OS(a2,sv)*pow(a1,sv*1.0)+pow(a1*a2,sv*1.0)/gsl_sf_fact(sv))-gsl_sf_lnfact(sv);
		if (log_b_part!=log_b_part || log_b_part==log(0.0)) log_b_part=log(pse_k_OS(b1,sv)*pow(b2,sv*1.0)+pse_k_OS(b2,sv)*pow(b1,sv*1.0)+pow(b1*b2,sv*1.0)/gsl_sf_fact(sv))-gsl_sf_lnfact(sv);
		w=log_a_part-log_b_part;
	}
	else if (fmin(fmax(a1,a2),fmax(b1,b2))>700){
		w=(a1+a2)-(b1+b2);
	}
	else{
		if (fmax(a1,a2)>fmax(b1,b2)){
			w=(a1+a2)-(b1+b2+log(1-gsl_cdf_poisson_P(sv-1,b1)*gsl_cdf_poisson_P(sv-1,b2)));
		}
		else{
			w=(a1+a2+log(1-gsl_cdf_poisson_P(sv-1,a1)*gsl_cdf_poisson_P(sv-1,a2)))-(b1+b2);
		}
	}
	return w;
}

int ran_discrete_OS(gsl_rng *r,double w[]){
	double sum_w=w[0]+w[1]+w[2]+w[3]+w[4]+w[5];
	double a=gsl_rng_uniform_pos(r)*sum_w;
	if (a<w[0]) return 0;
	else if (a<(w[0]+w[1])) return 1;
	else if (a<(w[0]+w[1]+w[2])) return 2;
	else if (a<(w[0]+w[1]+w[2]+w[3])) return 3;
	else if (a<(w[0]+w[1]+w[2]+w[3]+w[4])) return 4;
	else return 5;
}

int sample_z_j_OS(gsl_rng *r,double p_j[],double lambda0_j,double lambda1_j,double lambda2_j,double lambda3_j,double lambda4_j,double lambda5_j,double mu0_j,double mu2_j,double mu3_j,double mu4_j,double mu5_j,int x_j,int y_j,int sv){
	double w[6],max_w;
	w[0]=log(p_j[0]);
	w[1]=log(p_j[1])+x_j*(log(lambda1_j)-log(lambda0_j))+y_j*(log(lambda1_j)-log(mu0_j))+adjust_OS(lambda0_j,mu0_j,lambda1_j,lambda1_j,sv);
	w[2]=log(p_j[2])+x_j*(log(lambda2_j)-log(lambda0_j))+y_j*(log(mu2_j)-log(mu0_j))+adjust_OS(lambda0_j,mu0_j,lambda2_j,mu2_j,sv);
	w[3]=log(p_j[3])+x_j*(log(lambda3_j)-log(lambda0_j))+y_j*(log(mu3_j)-log(mu0_j))+adjust_OS(lambda0_j,mu0_j,lambda3_j,mu3_j,sv);
	w[4]=log(p_j[4])+x_j*(log(lambda4_j)-log(lambda0_j))+y_j*(log(mu4_j)-log(mu0_j))+adjust_OS(lambda0_j,mu0_j,lambda4_j,mu4_j,sv);
	w[5]=log(p_j[5])+x_j*(log(lambda5_j)-log(lambda0_j))+y_j*(log(mu5_j)-log(mu0_j))+adjust_OS(lambda0_j,mu0_j,lambda5_j,mu5_j,sv);
	max_w=fmax(fmax(fmax(w[0],w[1]),fmax(w[2],w[3])),fmax(w[4],w[5]));
	w[0]=exp(w[0]-max_w); w[1]=exp(w[1]-max_w); w[2]=exp(w[2]-max_w); w[3]=exp(w[3]-max_w); w[4]=exp(w[4]-max_w); w[5]=exp(w[5]-max_w);
	return   ran_discrete_OS(r,w);
}

void sample_p_j_OS(gsl_rng *r,int z_j,double p_j[6],double d,double w_j[6]){
	double c[6]={w_j[0]*d,w_j[1]*d,w_j[2]*d,w_j[3]*d,w_j[4]*d,w_j[5]*d};
	c[z_j]=c[z_j]+1.0;
        ran_dirichlet_OS(r,c,p_j);
}

Result sample_d_OS(gsl_rng *r,double d,int n,double **p,double **w,double proposal_sd){
	double d_old=d;
	double lb=fmax(d-proposal_sd,0),ub=fmin(d+proposal_sd,1);
	double d_new=gsl_rng_uniform_pos(r)*(ub-lb)+lb;

	double sum_w;

	double alpha=0.0;
	int i;
	for (i=0;i<n;i++){
		sum_w=w[i][0]+w[i][1]+w[i][2]+w[i][3]+w[i][4]+w[i][5];
		alpha=alpha+gsl_sf_lngamma(w[i][0]*d_old)-gsl_sf_lngamma(w[i][0]*d_new)
			+gsl_sf_lngamma(w[i][1]*d_old)-gsl_sf_lngamma(w[i][1]*d_new)
			+gsl_sf_lngamma(w[i][2]*d_old)-gsl_sf_lngamma(w[i][2]*d_new)
			+gsl_sf_lngamma(w[i][3]*d_old)-gsl_sf_lngamma(w[i][3]*d_new)
			+gsl_sf_lngamma(w[i][4]*d_old)-gsl_sf_lngamma(w[i][4]*d_new)
			+gsl_sf_lngamma(w[i][5]*d_old)-gsl_sf_lngamma(w[i][5]*d_new)
			+gsl_sf_lngamma(sum_w*d_new)-gsl_sf_lngamma(sum_w*d_old)	
			+(d_new-d_old)*(w[i][0]*log(p[i][0])+w[i][1]*log(p[i][1])+w[i][2]*log(p[i][2])
					+w[i][3]*log(p[i][3])+w[i][4]*log(p[i][4])+w[i][5]*log(p[i][5]));
	}
	alpha=alpha+log(ub-lb)-log(fmin(d_new+proposal_sd,1)-fmax(d_new-proposal_sd,0));

	Result rslt;
        if (log(gsl_rng_uniform_pos(r))<alpha){
                rslt.sample=d_new;
                rslt.accept=1;
        }
        else{
                rslt.sample=d_old;
                rslt.accept=0;
        }

        return rslt;
}

Result TL_accept_mean_j_OS(gsl_rng *r,double mean_j_new,double mean_j_old,int count_j,int z_j,int flag,double reference_j,double lc_j,double m,int sv){
	Result rslt;	

	double alpha=0.5*(mean_j_old+mean_j_new-2*lc_j)*(mean_j_old-mean_j_new)/m+log(mean_j_new)-log(mean_j_old);
	if (z_j==flag){
		if (flag==1){
			alpha=alpha+count_j*(log(mean_j_new)-log(mean_j_old))
				+adjust_OS(mean_j_old,mean_j_old,mean_j_new,mean_j_new,sv);
		}
		else{
			alpha=alpha+count_j*(log(mean_j_new)-log(mean_j_old))
				+adjust_OS(mean_j_old,reference_j,mean_j_new,reference_j,sv);
		}
	}
	else ;

	if (log(gsl_rng_uniform_pos(r))<alpha){
		rslt.sample=mean_j_new;
		rslt.accept=1;
	}
	else{
		rslt.sample=mean_j_old;
                rslt.accept=0;
	}

	return rslt;
}

Result sample_TL_mean_j_OS(gsl_rng *r,double mean_j,double reference_j,double lc_j,int x_j,int y_j,int z_j,int group,char treat,double m,int sv,double proposal_sd){
	double  mean_j_old=mean_j;
        double mean_j_new=gsl_ran_lognormal(r,log(mean_j),proposal_sd);

	Result rslt;

	if (mean_j_new<=lc_j){
                rslt.sample=mean_j_old;
                rslt.accept=0;
        }
	else{
		if (group==0){
			if (treat=='x'){
				if (mean_j_new<=reference_j){
					rslt.sample=mean_j_old;
					rslt.accept=0;
				}
				else rslt=TL_accept_mean_j_OS(r,mean_j_new,mean_j_old,x_j,z_j,0,reference_j,lc_j,m,sv);
			}
			else{
				if (mean_j_new>=reference_j){
                                        rslt.sample=mean_j_old;
                                        rslt.accept=0;
                                }
                                else rslt=TL_accept_mean_j_OS(r,mean_j_new,mean_j_old,y_j,z_j,0,reference_j,lc_j,m,sv);
			}
		}					
		else if (group==1){
			rslt=TL_accept_mean_j_OS(r,mean_j_new,mean_j_old,x_j+y_j,z_j,1,reference_j,lc_j,m,sv);
		}
		else if (group==2){
			if (treat=='x'){
                                if (mean_j_new>=reference_j){
                                        rslt.sample=mean_j_old;
                                        rslt.accept=0;
                                }
                                else rslt=TL_accept_mean_j_OS(r,mean_j_new,mean_j_old,x_j,z_j,2,reference_j,lc_j,m,sv);
                        }
                        else{
                                if (mean_j_new<=reference_j){
                                        rslt.sample=mean_j_old;
                                        rslt.accept=0;
                                }
                                else rslt=TL_accept_mean_j_OS(r,mean_j_new,mean_j_old,y_j,z_j,2,reference_j,lc_j,m,sv);
                        }
                }
		else if (group==3){
			rslt=TL_accept_mean_j_OS(r,mean_j_new,mean_j_old,x_j,z_j,3,reference_j,lc_j,m,sv);
		}
		else rslt=TL_accept_mean_j_OS(r,mean_j_new,mean_j_old,y_j,z_j,4,reference_j,lc_j,m,sv);
	}		

	return rslt;
}

Result FL_accept_mean_j_OS(gsl_rng *r,double mean_j_new,double mean_j_old,int count_j,int z_j,int flag,double reference_j,double b,double r0,double ajst_j,int sv){
	Result rslt;	

	double alpha;
	if (z_j==flag){
		alpha=(b+count_j)*(log(mean_j_new)-log(mean_j_old))
                	-(b/r0)*(mean_j_new-mean_j_old)+adjust_OS(mean_j_old,reference_j,mean_j_new,reference_j,sv);
	}
	else alpha=b*(log(mean_j_new)-log(mean_j_old))-(b/r0)*(mean_j_new-mean_j_old);
	alpha=alpha+ajst_j;

	if (log(gsl_rng_uniform_pos(r))<alpha){
		rslt.sample=mean_j_new;
		rslt.accept=1;
	}
	else{
		rslt.sample=mean_j_old;
                rslt.accept=0;
	}

	return rslt;
}

/* order of mean_j,mean1_j,mean2_j,mean3_j in the following function is important, should be one from {mu3,lambda4,lambda5,mu5}, followed by the rest with the same order */
Result sample_FL_mean_j_OS(gsl_rng *r,double mean_j,double mean1_j,double mean2_j,double mean3_j,double lambda0_j,double mu0_j,double lambda1_j,double lambda2_j,double mu2_j,double lambda3_j,double mu4_j,int x_j,int y_j,int z_j,int group,char treat,double m,double b,double r0,int sv,double proposal_sd){
	double  mean_j_old=mean_j;
        double mean_j_new=gsl_ran_lognormal(r,log(mean_j),proposal_sd);

	Result rslt;
	double max_old_j,max_new_j,ajst_j;
	double constraint_j=fmin(fmin(fmin(lambda1_j,lambda2_j),fmin(mu0_j,mu4_j)),lambda3_j);

        if (mean_j_new>=constraint_j){
                rslt.sample=mean_j_old;
                rslt.accept=0;
        }	
	else{
		max_old_j=fmax(fmax(mean_j_old,mean1_j),fmax(mean2_j,mean3_j));
		max_new_j=fmax(fmax(mean_j_new,mean1_j),fmax(mean2_j,mean3_j));		
		if (max_old_j==max_new_j){
			ajst_j=0.0;
		}
		else{
			ajst_j=-0.5*(max_old_j-max_new_j)*(2*(lambda0_j+mu0_j+lambda1_j+lambda2_j+mu2_j+lambda3_j+mu4_j)-7*(max_new_j+max_old_j))/m;
		}
		if (group==3){
			rslt=FL_accept_mean_j_OS(r,mean_j_new,mean_j_old,y_j,z_j,3,lambda3_j,b,r0,ajst_j,sv);
		}
		else if (group==4){
			rslt=FL_accept_mean_j_OS(r,mean_j_new,mean_j_old,x_j,z_j,4,mu4_j,b,r0,ajst_j,sv);
		}
		else{
			if (treat=='x'){
				rslt=FL_accept_mean_j_OS(r,mean_j_new,mean_j_old,x_j,z_j,5,mean3_j,b,r0,ajst_j,sv);
			}
			else{
				rslt=FL_accept_mean_j_OS(r,mean_j_new,mean_j_old,y_j,z_j,5,mean3_j,b,r0,ajst_j,sv);
			}
		}	
	}

	return rslt;
}	

double sample_m_OS(gsl_rng *r,int const VUB,int n,double sum_lambdadiff_sq){
	double w=VUB+1.0;
	while (w>=(VUB*1.0)){
		w=1/gsl_ran_gamma(r,3.5*n-0.5,2.0/sum_lambdadiff_sq);
	}
	return w;
}

Result sample_r0_OS(gsl_rng *r,int const UUB,double r0,double b,int n,double sum_parm,double proposal_sd){
	double r0_old=r0;
	double r0_new=gsl_ran_lognormal(r,log(r0),proposal_sd);

	Result rslt;
	double alpha;
	if (r0_new>=UUB){
		rslt.sample=r0_old;
		rslt.accept=0;
	}
	else {
		alpha=(4*n*b-1)*(log(r0_old)-log(r0_new))+b*sum_parm*(1/r0_old-1/r0_new);
		if (log(gsl_rng_uniform_pos(r))<alpha){
                	rslt.sample=r0_new;
                	rslt.accept=1;
       		}
        	else{
                	rslt.sample=r0_old;
                	rslt.accept=0;
        	}
	}
        return rslt;
}
		
Result sample_b_OS(gsl_rng *r,int const UUB,double b,double r0,int n,double sum_parm,double sum_log_parm,double proposal_sd){
	double b_old=b;
	double b_new=gsl_ran_lognormal(r,log(b),proposal_sd);
	
	Result rslt;
	double alpha;
        if (b_new>=UUB){
                rslt.sample=b_old;
                rslt.accept=0;
        }
	else{
		alpha=4*n*(log(b_new)*b_new-log(b_old)*b_old)+(b_new-b_old)*(sum_log_parm-sum_parm/r0-4*n*log(r0))
			-4*n*(gsl_sf_lngamma(b_new)-gsl_sf_lngamma(b_old))+log(b_new)-log(b_old);
		if (log(gsl_rng_uniform_pos(r))<alpha){
                        rslt.sample=b_new;
                        rslt.accept=1;
                }
                else{
                        rslt.sample=b_old;
                        rslt.accept=0;
                }
	}
	return rslt;
}


/* find number of maximum values among 6 double values */
int unique_max_OS(double m,double v0,double v1,double v2,double v3,double v4,double v5){
	double v[6]={v0,v1,v2,v3,v4,v5};
	int i,k=0;
	
	for (i=0;i<6;i++){
		if (m==v[i]) k++;
	}
	
	return k;
}




/* main function */


void MDOS(
	/* nxy=(n_obs,x,y) */
	int *nxy,

	/* wp=(w0,w1,w2,w3,w4,w5) */
	double *wp,

	/* model_p=(sv,UUB,DUB,VUB) */ 
	int *model_p,

	/* running_p=(iter_all,thin,burnin,r_seed) */
	int *running_p,

	int *initial_z,

	/* initial_p=(initial_lambda0,...,initial_lambda5,initial_mu0,initial_mu2,...,initial_mu5,initial_p0,...,initial_p5) */
	double *initial_p,

	/* initial_h=(d,m,r0,b) */
	double *initial_h,

	/* proposal_sd=(d,r0,b,lambda0,...,mu5) */
	double *proposal_sd,

	int *obs,

	/* post_prob_z=(post_prob_z_0,...,post_prob_z_5) */
	double *post_prob_z,int *bd,

	/* hp_accpt=(r0,b,d,lambda0,...,mu5) */
	double *hp_accpt,

	/* h_samples=(d,m,r0,b) */
	double *h_samples,

	double *p0_samples_1,double *p1_samples_1,double *p2_samples_1,double *p3_samples_1,double *p4_samples_1,double *p5_samples_1,
	double *lambda0_samples_1,double *lambda1_samples_1,double *lambda2_samples_1,double *lambda3_samples_1,double *lambda4_samples_1,double *lambda5_samples_1,
	double *mu0_samples_1,double *mu2_samples_1,double *mu3_samples_1,double *mu4_samples_1,double *mu5_samples_1,


	double *p0_samples_2,double *p1_samples_2,double *p2_samples_2,double *p3_samples_2,double *p4_samples_2,double *p5_samples_2,
	double *lambda0_samples_2,double *lambda1_samples_2,double *lambda2_samples_2,double *lambda3_samples_2,double *lambda4_samples_2,double *lambda5_samples_2,
	double *mu0_samples_2,double *mu2_samples_2,double *mu3_samples_2,double *mu4_samples_2,double *mu5_samples_2,


	double *p0_samples_3,double *p1_samples_3,double *p2_samples_3,double *p3_samples_3,double *p4_samples_3,double *p5_samples_3,
	double *lambda0_samples_3,double *lambda1_samples_3,double *lambda2_samples_3,double *lambda3_samples_3,double *lambda4_samples_3,double *lambda5_samples_3,
	double *mu0_samples_3,double *mu2_samples_3,double *mu3_samples_3,double *mu4_samples_3,double *mu5_samples_3,

	time_t *run_time){

	/* get start time */
	time_t start=time(0);

	/* find datasize */
        int n=nxy[0],i;
	
	/* read in weight information */
	double **w=allocate2dDoubleArray(n,6);

	for (i=0;i<n;i++){
		w[i][0]=wp[i]*model_p[2]; w[i][1]=wp[n+i]*model_p[2]; w[i][2]=wp[2*n+i]*model_p[2];
		w[i][3]=wp[3*n+i]*model_p[2]; w[i][4]=wp[4*n+i]*model_p[2]; w[i][5]=wp[5*n+i]*model_p[2];
	}

	/* define variables for updating and set initial values */
	int *z_post=(int *)malloc(n*sizeof(int));
	double *lambda0_post=(double *)malloc(n*sizeof(double)),*lambda1_post=(double *)malloc(n*sizeof(double));
	double *lambda2_post=(double *)malloc(n*sizeof(double)),*lambda3_post=(double *)malloc(n*sizeof(double));
	double *lambda4_post=(double *)malloc(n*sizeof(double)),*lambda5_post=(double *)malloc(n*sizeof(double));
	double *mu0_post=(double *)malloc(n*sizeof(double)),*mu2_post=(double *)malloc(n*sizeof(double));
	double *mu3_post=(double *)malloc(n*sizeof(double)),*mu4_post=(double *)malloc(n*sizeof(double)),*mu5_post=(double *)malloc(n*sizeof(double));
	double **p_post=allocate2dDoubleArray(n,6);
	for (i=0;i<n;i++){
		z_post[i]=initial_z[i];
		lambda0_post[i]=initial_p[i]; lambda1_post[i]=initial_p[n+i];
		lambda2_post[i]=initial_p[2*n+i]; lambda3_post[i]=initial_p[3*n+i]; lambda4_post[i]=initial_p[4*n+i]; lambda5_post[i]=initial_p[5*n+i];
		mu0_post[i]=initial_p[6*n+i]; mu2_post[i]=initial_p[7*n+i]; mu3_post[i]=initial_p[8*n+i]; mu4_post[i]=initial_p[9*n+i]; mu5_post[i]=initial_p[10*n+i];
		p_post[i][0]=initial_p[11*n+i]; p_post[i][1]=initial_p[12*n+i]; p_post[i][2]=initial_p[13*n+i]; 
		p_post[i][3]=initial_p[14*n+i]; p_post[i][4]=initial_p[15*n+i]; p_post[i][5]=initial_p[16*n+i];
	}

	double d_post=initial_h[0],m_post=initial_h[1],r0_post=initial_h[2],b_post=initial_h[3];
	
	/* define storage variables and set proposal sd's */
	double accept_d=0.0,accept_r0=0.0,accept_b=0.0; 
	
	double *accept_lambda0=(double *)malloc(n*sizeof(double)),*accept_mu0=(double *)malloc(n*sizeof(double));
	double *accept_lambda1=(double *)malloc(n*sizeof(double)),*accept_lambda2=(double *)malloc(n*sizeof(double)),*accept_mu2=(double *)malloc(n*sizeof(double));
	double *accept_lambda3=(double *)malloc(n*sizeof(double)),*accept_mu4=(double *)malloc(n*sizeof(double));
	double *accept_mu3=(double *)malloc(n*sizeof(double)),*accept_lambda4=(double *)malloc(n*sizeof(double));
	double *accept_lambda5=(double *)malloc(n*sizeof(double)),*accept_mu5=(double *)malloc(n*sizeof(double));
	for (i=0;i<n;i++){
		accept_lambda0[i]=0.0; accept_mu0[i]=0.0; accept_lambda1[i]=0.0; accept_lambda2[i]=0.0; accept_mu2[i]=0.0; accept_lambda3[i]=0.0; accept_mu4[i]=0.0;
		accept_mu3[i]=0.0; accept_lambda4[i]=0.0; accept_lambda5[i]=0.0; accept_mu5[i]=0.0;
	}
		
	/* define additional variables */
	int j,k=0,n_samples=running_p[0]/running_p[1];
	Result tmp;
	double max_FL_j;
	double *sum_post_prob_z_0=(double *)malloc(n*sizeof(double)),*sum_post_prob_z_1=(double *)malloc(n*sizeof(double));
	double *sum_post_prob_z_2=(double *)malloc(n*sizeof(double)),*sum_post_prob_z_3=(double *)malloc(n*sizeof(double)),*sum_post_prob_z_4=(double *)malloc(n*sizeof(double));
	double wt0,wt1,wt2,wt3,wt4,wt5,max_w;
	double uc_j,lc_j;
	double sum_diff_sq,sum_parm_0,sum_log_parm_0;
	for (i=0;i<n;i++){
		sum_post_prob_z_0[i]=0.0; sum_post_prob_z_1[i]=0.0;
		sum_post_prob_z_2[i]=0.0; sum_post_prob_z_3[i]=0.0; sum_post_prob_z_4[i]=0.0;
	}

	/* start Gibbs sampler */
	/* set random generator and seed */
	gsl_rng *r;
	r=gsl_rng_alloc(gsl_rng_mt19937);
	gsl_rng_set(r,running_p[3]);	

	/* update parameters */
	for (i=0;i<running_p[0];i++){

		sum_diff_sq=0.0;
		sum_parm_0=0.0; sum_log_parm_0=0.0;
	
		for (j=0;j<n;j++){

			z_post[j]=sample_z_j_OS(r,p_post[j],lambda0_post[j],lambda1_post[j],lambda2_post[j],lambda3_post[j],lambda4_post[j],lambda5_post[j],mu0_post[j],mu2_post[j],mu3_post[j],mu4_post[j],mu5_post[j],nxy[1+j],nxy[1+n+j],model_p[0]);

			sample_p_j_OS(r,z_post[j],p_post[j],d_post,w[j]);
	
			lc_j=fmax(fmax(mu3_post[j],lambda4_post[j]),fmax(lambda5_post[j],mu5_post[j]));			
			tmp=sample_TL_mean_j_OS(r,lambda0_post[j],mu0_post[j],lc_j,nxy[1+j],nxy[1+n+j],z_post[j],0,'x',m_post,model_p[0],proposal_sd[3]);
			lambda0_post[j]=tmp.sample;
			accept_lambda0[j]=accept_lambda0[j]+tmp.accept;

			tmp=sample_TL_mean_j_OS(r,mu0_post[j],lambda0_post[j],lc_j,nxy[1+j],nxy[1+n+j],z_post[j],0,'y',m_post,model_p[0],proposal_sd[9]);
                        mu0_post[j]=tmp.sample;
                        accept_mu0[j]=accept_mu0[j]+tmp.accept;
		
			tmp=sample_TL_mean_j_OS(r,lambda1_post[j],lambda1_post[j],lc_j,nxy[1+j],nxy[1+n+j],z_post[j],1,'x',m_post,model_p[0],proposal_sd[4]);
			lambda1_post[j]=tmp.sample;
                        accept_lambda1[j]=accept_lambda1[j]+tmp.accept;

			tmp=sample_TL_mean_j_OS(r,lambda2_post[j],mu2_post[j],lc_j,nxy[1+j],nxy[1+n+j],z_post[j],2,'x',m_post,model_p[0],proposal_sd[5]);
			lambda2_post[j]=tmp.sample;
                        accept_lambda2[j]=accept_lambda2[j]+tmp.accept;

			tmp=sample_TL_mean_j_OS(r,mu2_post[j],lambda2_post[j],lc_j,nxy[1+j],nxy[1+n+j],z_post[j],2,'y',m_post,model_p[0],proposal_sd[10]);
			mu2_post[j]=tmp.sample;
                        accept_mu2[j]=accept_mu2[j]+tmp.accept;

			tmp=sample_TL_mean_j_OS(r,lambda3_post[j],mu3_post[j],lc_j,nxy[1+j],nxy[1+n+j],z_post[j],3,'x',m_post,model_p[0],proposal_sd[6]);
			lambda3_post[j]=tmp.sample;
                        accept_lambda3[j]=accept_lambda3[j]+tmp.accept;	

			tmp=sample_TL_mean_j_OS(r,mu4_post[j],lambda4_post[j],lc_j,nxy[1+j],nxy[1+n+j],z_post[j],4,'y',m_post,model_p[0],proposal_sd[12]);
			mu4_post[j]=tmp.sample;
                        accept_mu4[j]=accept_mu4[j]+tmp.accept;
	

			uc_j=fmin(fmin(fmin(lambda1_post[j],lambda2_post[j]),fmin(mu0_post[j],mu4_post[j])),lambda3_post[j]);
			tmp=sample_FL_mean_j_OS(r,lambda4_post[j],mu3_post[j],lambda5_post[j],mu5_post[j],lambda0_post[j],mu0_post[j],lambda1_post[j],lambda2_post[j],mu2_post[j],lambda3_post[j],mu4_post[j],nxy[1+j],nxy[1+n+j],z_post[j],4,'x',m_post,b_post,r0_post,model_p[0],proposal_sd[7]);
                        lambda4_post[j]=tmp.sample;
                        accept_lambda4[j]=accept_lambda4[j]+tmp.accept;

                        tmp=sample_FL_mean_j_OS(r,lambda5_post[j],mu3_post[j],lambda4_post[j],mu5_post[j],lambda0_post[j],mu0_post[j],lambda1_post[j],lambda2_post[j],mu2_post[j],lambda3_post[j],mu4_post[j],nxy[1+j],nxy[1+n+j],z_post[j],5,'x',m_post,b_post,r0_post,model_p[0],proposal_sd[8]);
                        lambda5_post[j]=tmp.sample;
                        accept_lambda5[j]=accept_lambda5[j]+tmp.accept;			

			tmp=sample_FL_mean_j_OS(r,mu3_post[j],lambda4_post[j],lambda5_post[j],mu5_post[j],lambda0_post[j],mu0_post[j],lambda1_post[j],lambda2_post[j],mu2_post[j],lambda3_post[j],mu4_post[j],nxy[1+j],nxy[1+n+j],z_post[j],3,'y',m_post,b_post,r0_post,model_p[0],proposal_sd[11]);
                        mu3_post[j]=tmp.sample;
                        accept_mu3[j]=accept_mu3[j]+tmp.accept;

                        tmp=sample_FL_mean_j_OS(r,mu5_post[j],mu3_post[j],lambda4_post[j],lambda5_post[j],lambda0_post[j],mu0_post[j],lambda1_post[j],lambda2_post[j],mu2_post[j],lambda3_post[j],mu4_post[j],nxy[1+j],nxy[1+n+j],z_post[j],5,'y',m_post,b_post,r0_post,model_p[0],proposal_sd[13]);
                        mu5_post[j]=tmp.sample;
                        accept_mu5[j]=accept_mu5[j]+tmp.accept;


			max_FL_j=fmax(fmax(mu3_post[j],lambda4_post[j]),fmax(lambda5_post[j],mu5_post[j]));
			sum_diff_sq=sum_diff_sq+pow(lambda0_post[j]-max_FL_j,2.0)+pow(mu0_post[j]-max_FL_j,2.0)+pow(lambda1_post[j]-max_FL_j,2.0)+pow(lambda2_post[j]-max_FL_j,2.0)+pow(mu2_post[j]-max_FL_j,2.0)+pow(lambda3_post[j]-max_FL_j,2.0)+pow(mu4_post[j]-max_FL_j,2.0);
			sum_parm_0=sum_parm_0+mu3_post[j]+lambda4_post[j]+lambda5_post[j]+mu5_post[j];
			sum_log_parm_0=sum_log_parm_0+log(mu3_post[j])+log(lambda4_post[j])+log(lambda5_post[j])+log(mu5_post[j]);	 


			if ((fmod(i+1,running_p[1])==0) && (i>(running_p[2]-1))){
				wt0=log(p_post[j][0]);
				wt1=log(p_post[j][1])
					+nxy[1+j]*(log(lambda1_post[j])-log(lambda0_post[j]))+nxy[1+n+j]*(log(lambda1_post[j])-log(mu0_post[j]))
					+adjust_OS(lambda0_post[j],mu0_post[j],lambda1_post[j],lambda1_post[j],model_p[0]);
				wt2=log(p_post[j][2])
					+nxy[1+j]*(log(lambda2_post[j])-log(lambda0_post[j]))+nxy[1+n+j]*(log(mu2_post[j])-log(mu0_post[j]))
					+adjust_OS(lambda0_post[j],mu0_post[j],lambda2_post[j],mu2_post[j],model_p[0]);
				wt3=log(p_post[j][3])
					+nxy[1+j]*(log(lambda3_post[j])-log(lambda0_post[j]))+nxy[1+n+j]*(log(mu3_post[j])-log(mu0_post[j]))
                                        +adjust_OS(lambda0_post[j],mu0_post[j],lambda3_post[j],mu3_post[j],model_p[0]);
				wt4=log(p_post[j][4])
                                        +nxy[1+j]*(log(lambda4_post[j])-log(lambda0_post[j]))+nxy[1+n+j]*(log(mu4_post[j])-log(mu0_post[j]))
                                        +adjust_OS(lambda0_post[j],mu0_post[j],lambda4_post[j],mu4_post[j],model_p[0]);
				wt5=log(p_post[j][5])
                                        +nxy[1+j]*(log(lambda5_post[j])-log(lambda0_post[j]))+nxy[1+n+j]*(log(mu5_post[j])-log(mu0_post[j]))
                                        +adjust_OS(lambda0_post[j],mu0_post[j],lambda5_post[j],mu5_post[j],model_p[0]);
				max_w=fmax(fmax(fmax(wt0,wt1),fmax(wt2,wt3)),fmax(wt4,wt5));
				wt0=exp(wt0-max_w); wt1=exp(wt1-max_w); wt2=exp(wt2-max_w); wt3=exp(wt3-max_w); wt4=exp(wt4-max_w); wt5=exp(wt5-max_w);
                                sum_post_prob_z_0[j]=sum_post_prob_z_0[j]+wt0/(wt0+wt1+wt2+wt3+wt4+wt5);
				sum_post_prob_z_1[j]=sum_post_prob_z_1[j]+wt1/(wt0+wt1+wt2+wt3+wt4+wt5);
				sum_post_prob_z_2[j]=sum_post_prob_z_2[j]+wt2/(wt0+wt1+wt2+wt3+wt4+wt5);
                                sum_post_prob_z_3[j]=sum_post_prob_z_3[j]+wt3/(wt0+wt1+wt2+wt3+wt4+wt5);
				sum_post_prob_z_4[j]=sum_post_prob_z_4[j]+wt4/(wt0+wt1+wt2+wt3+wt4+wt5);
			}
		}
		
		tmp=sample_d_OS(r,d_post,n,p_post,w,proposal_sd[0]);
		d_post=tmp.sample;
		accept_d=accept_d+tmp.accept;
	
		m_post=sample_m_OS(r,model_p[3],n,sum_diff_sq);

		tmp=sample_r0_OS(r,model_p[1],r0_post,b_post,n,sum_parm_0,proposal_sd[1]);
                r0_post=tmp.sample;
                accept_r0=accept_r0+tmp.accept;

                tmp=sample_b_OS(r,model_p[1],b_post,r0_post,n,sum_parm_0,sum_log_parm_0,proposal_sd[2]);
                b_post=tmp.sample;
                accept_b=accept_b+tmp.accept;

		if (fmod(i+1,running_p[1])==0){
			h_samples[k]=d_post;
			h_samples[n_samples+k]=m_post;
			h_samples[2*n_samples+k]=r0_post;
			h_samples[3*n_samples+k]=b_post;

			p0_samples_1[k]=p_post[obs[0]-1][0];
			p1_samples_1[k]=p_post[obs[0]-1][1];
			p2_samples_1[k]=p_post[obs[0]-1][2];
			p3_samples_1[k]=p_post[obs[0]-1][3];
			p4_samples_1[k]=p_post[obs[0]-1][4];
			p5_samples_1[k]=p_post[obs[0]-1][5];
			lambda0_samples_1[k]=lambda0_post[obs[0]-1];
			lambda1_samples_1[k]=lambda1_post[obs[0]-1];
			lambda2_samples_1[k]=lambda2_post[obs[0]-1];
			lambda3_samples_1[k]=lambda3_post[obs[0]-1];
			lambda4_samples_1[k]=lambda4_post[obs[0]-1];
			lambda5_samples_1[k]=lambda5_post[obs[0]-1];
			mu0_samples_1[k]=mu0_post[obs[0]-1];
			mu2_samples_1[k]=mu2_post[obs[0]-1];
			mu3_samples_1[k]=mu3_post[obs[0]-1];
			mu4_samples_1[k]=mu4_post[obs[0]-1];
			mu5_samples_1[k]=mu5_post[obs[0]-1];

			p0_samples_2[k]=p_post[obs[1]-1][0];
			p1_samples_2[k]=p_post[obs[1]-1][1];
			p2_samples_2[k]=p_post[obs[1]-1][2];
			p3_samples_2[k]=p_post[obs[1]-1][3];
			p4_samples_2[k]=p_post[obs[1]-1][4];
			p5_samples_2[k]=p_post[obs[1]-1][5];
			lambda0_samples_2[k]=lambda0_post[obs[1]-1];
			lambda1_samples_2[k]=lambda1_post[obs[1]-1];
			lambda2_samples_2[k]=lambda2_post[obs[1]-1];
			lambda3_samples_2[k]=lambda3_post[obs[1]-1];
			lambda4_samples_2[k]=lambda4_post[obs[1]-1];
			lambda5_samples_2[k]=lambda5_post[obs[1]-1];
			mu0_samples_2[k]=mu0_post[obs[1]-1];
			mu2_samples_2[k]=mu2_post[obs[1]-1];
			mu3_samples_2[k]=mu3_post[obs[1]-1];
			mu4_samples_2[k]=mu4_post[obs[1]-1];
			mu5_samples_2[k]=mu5_post[obs[1]-1];


			p0_samples_3[k]=p_post[obs[2]-1][0];
			p1_samples_3[k]=p_post[obs[2]-1][1];
			p2_samples_3[k]=p_post[obs[2]-1][2];
			p3_samples_3[k]=p_post[obs[2]-1][3];
			p4_samples_3[k]=p_post[obs[2]-1][4];
			p5_samples_3[k]=p_post[obs[2]-1][5];
			lambda0_samples_3[k]=lambda0_post[obs[2]-1];
			lambda1_samples_3[k]=lambda1_post[obs[2]-1];
			lambda2_samples_3[k]=lambda2_post[obs[2]-1];
			lambda3_samples_3[k]=lambda3_post[obs[2]-1];
			lambda4_samples_3[k]=lambda4_post[obs[2]-1];
			lambda5_samples_3[k]=lambda5_post[obs[2]-1];
			mu0_samples_3[k]=mu0_post[obs[2]-1];
			mu2_samples_3[k]=mu2_post[obs[2]-1];
			mu3_samples_3[k]=mu3_post[obs[2]-1];
			mu4_samples_3[k]=mu4_post[obs[2]-1];
			mu5_samples_3[k]=mu5_post[obs[2]-1];

			k=k+1;
		}
	
	}


	/* free the random number generator */
	gsl_rng_free(r);
	
	/* output posterior probabilities for z_j=0, 1, 2, 3, 4 or 5, and bayesian decision*/
	double max_p_j;

        for (j=0;j<n;j++){
		post_prob_z[j]=sum_post_prob_z_0[j]/((running_p[0]-running_p[2])/(running_p[1]));
		post_prob_z[n+j]=sum_post_prob_z_1[j]/((running_p[0]-running_p[2])/(running_p[1]));
		post_prob_z[2*n+j]=sum_post_prob_z_2[j]/((running_p[0]-running_p[2])/(running_p[1]));
                post_prob_z[3*n+j]=sum_post_prob_z_3[j]/((running_p[0]-running_p[2])/(running_p[1]));
		post_prob_z[4*n+j]=sum_post_prob_z_4[j]/((running_p[0]-running_p[2])/(running_p[1]));
                post_prob_z[5*n+j]=fmax(0.0,1.0-post_prob_z[j]-post_prob_z[n+j]-post_prob_z[2*n+j]-post_prob_z[3*n+j]-post_prob_z[4*n+j]);
                max_p_j=fmax(fmax(fmax(post_prob_z[j],post_prob_z[n+j]),fmax(post_prob_z[2*n+j],post_prob_z[3*n+j])),fmax(post_prob_z[4*n+j],post_prob_z[5*n+j]));
                if (unique_max_OS(max_p_j,post_prob_z[j],post_prob_z[n+j],post_prob_z[2*n+j],post_prob_z[3*n+j],post_prob_z[4*n+j],post_prob_z[5*n+j])>1){
                        bd[j]=-1;
                }
                else{
                        if (post_prob_z[j]==max_p_j) bd[j]=0;
                        else if (post_prob_z[n+j]==max_p_j) bd[j]=1;
                        else if (post_prob_z[2*n+j]==max_p_j) bd[j]=2;
                        else if (post_prob_z[3*n+j]==max_p_j) bd[j]=3;
                        else if (post_prob_z[4*n+j]==max_p_j) bd[j]=4;
                        else bd[j]=5;
                }
        }

	/* output the acceptance rates for r0 and b */
        hp_accpt[0]=accept_r0/(running_p[0]); hp_accpt[1]=accept_b/(running_p[0]); hp_accpt[2]=accept_d/(running_p[0]);

	/* output the acceptance rates for lambda's and mu's */
	double lambda0_accpt_lb=accept_lambda0[0],lambda0_accpt_ub=accept_lambda0[0];
	double lambda1_accpt_lb=accept_lambda1[0],lambda1_accpt_ub=accept_lambda1[0];
	double lambda2_accpt_lb=accept_lambda2[0],lambda2_accpt_ub=accept_lambda2[0];
	double lambda3_accpt_lb=accept_lambda3[0],lambda3_accpt_ub=accept_lambda3[0];
	double lambda4_accpt_lb=accept_lambda4[0],lambda4_accpt_ub=accept_lambda4[0];
	double lambda5_accpt_lb=accept_lambda5[0],lambda5_accpt_ub=accept_lambda5[0];

	double mu0_accpt_lb=accept_mu0[0],mu0_accpt_ub=accept_mu0[0];
	double mu2_accpt_lb=accept_mu2[0],mu2_accpt_ub=accept_mu2[0];
	double mu3_accpt_lb=accept_mu3[0],mu3_accpt_ub=accept_mu3[0];
	double mu4_accpt_lb=accept_mu4[0],mu4_accpt_ub=accept_mu4[0];
	double mu5_accpt_lb=accept_mu5[0],mu5_accpt_ub=accept_mu5[0];

	for (j=0;j<(n-1);j++){
		lambda0_accpt_lb=fmin(lambda0_accpt_lb,accept_lambda0[j+1]); lambda0_accpt_ub=fmax(lambda0_accpt_ub,accept_lambda0[j+1]);
		lambda1_accpt_lb=fmin(lambda1_accpt_lb,accept_lambda1[j+1]); lambda1_accpt_ub=fmax(lambda1_accpt_ub,accept_lambda1[j+1]);
		lambda2_accpt_lb=fmin(lambda2_accpt_lb,accept_lambda2[j+1]); lambda2_accpt_ub=fmax(lambda2_accpt_ub,accept_lambda2[j+1]);
		lambda3_accpt_lb=fmin(lambda3_accpt_lb,accept_lambda3[j+1]); lambda3_accpt_ub=fmax(lambda3_accpt_ub,accept_lambda3[j+1]);
		lambda4_accpt_lb=fmin(lambda4_accpt_lb,accept_lambda4[j+1]); lambda4_accpt_ub=fmax(lambda4_accpt_ub,accept_lambda4[j+1]);
		lambda5_accpt_lb=fmin(lambda5_accpt_lb,accept_lambda5[j+1]); lambda5_accpt_ub=fmax(lambda5_accpt_ub,accept_lambda5[j+1]);

		mu0_accpt_lb=fmin(mu0_accpt_lb,accept_mu0[j+1]); mu0_accpt_ub=fmax(mu0_accpt_ub,accept_mu0[j+1]);
		mu2_accpt_lb=fmin(mu2_accpt_lb,accept_mu2[j+1]); mu2_accpt_ub=fmax(mu2_accpt_ub,accept_mu2[j+1]);
		mu3_accpt_lb=fmin(mu3_accpt_lb,accept_mu3[j+1]); mu3_accpt_ub=fmax(mu3_accpt_ub,accept_mu3[j+1]);
		mu4_accpt_lb=fmin(mu4_accpt_lb,accept_mu4[j+1]); mu4_accpt_ub=fmax(mu4_accpt_ub,accept_mu4[j+1]);
		mu5_accpt_lb=fmin(mu5_accpt_lb,accept_mu5[j+1]); mu5_accpt_ub=fmax(mu5_accpt_ub,accept_mu5[j+1]);
	}

	hp_accpt[3]=lambda0_accpt_lb/(running_p[0]); hp_accpt[4]=lambda0_accpt_ub/(running_p[0]);
	hp_accpt[5]=lambda1_accpt_lb/(running_p[0]); hp_accpt[6]=lambda1_accpt_ub/(running_p[0]);
	hp_accpt[7]=lambda2_accpt_lb/(running_p[0]); hp_accpt[8]=lambda2_accpt_ub/(running_p[0]);
	hp_accpt[9]=lambda3_accpt_lb/(running_p[0]); hp_accpt[10]=lambda3_accpt_ub/(running_p[0]);
	hp_accpt[11]=lambda4_accpt_lb/(running_p[0]); hp_accpt[12]=lambda4_accpt_ub/(running_p[0]);
	hp_accpt[13]=lambda5_accpt_lb/(running_p[0]); hp_accpt[14]=lambda5_accpt_ub/(running_p[0]);

	hp_accpt[15]=mu0_accpt_lb/(running_p[0]); hp_accpt[16]=mu0_accpt_ub/(running_p[0]);
	hp_accpt[17]=mu2_accpt_lb/(running_p[0]); hp_accpt[18]=mu2_accpt_ub/(running_p[0]);
	hp_accpt[19]=mu3_accpt_lb/(running_p[0]); hp_accpt[20]=mu3_accpt_ub/(running_p[0]);
	hp_accpt[21]=mu4_accpt_lb/(running_p[0]); hp_accpt[22]=mu4_accpt_ub/(running_p[0]);
	hp_accpt[23]=mu5_accpt_lb/(running_p[0]); hp_accpt[24]=mu5_accpt_ub/(running_p[0]);

	/* free pointers */
	deallocate2dDoubleArray(w,n); deallocate2dDoubleArray(p_post,n);
	free(z_post); free(lambda0_post); free(lambda1_post); free(lambda2_post); free(lambda3_post); free(lambda4_post); free(lambda5_post);
	free(mu0_post); free(mu2_post); free(mu3_post); free(mu4_post); free(mu5_post);	
	free(accept_lambda0); free(accept_mu0); free(accept_lambda1);
	free(accept_lambda2); free(accept_mu2);	free(accept_lambda3); free(accept_mu3);
	free(accept_lambda4); free(accept_mu4); free(accept_lambda5); free(accept_mu5);
	free(sum_post_prob_z_0); free(sum_post_prob_z_1); free(sum_post_prob_z_2); free(sum_post_prob_z_3); free(sum_post_prob_z_4); 

	/* get runing time */
	time_t end=time(0);
	run_time[0]=(end-start);
}
